/*
This source file is part of ProceduralGenerator (https://sourceforge.net/projects/proceduralgene/)

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU Lesser General Public License as published by the Free Software
Foundation; either version 2 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License along with
the program; if not, write to the Free Software Foundation, Inc., 59 Temple
Place - Suite 330, Boston, MA 02111-1307, USA, or go to
http://www.gnu.org/copyleft/lesser.txt.
*/
#ifndef ___PG_Prerequisites___
#define ___PG_Prerequisites___

#if defined( _WIN32 )
#	if !defined( Generator_SHARED )
#		define GeneratorAPI
#	else
#		if defined( Generator_EXPORTS )
#			define GeneratorAPI __declspec( dllexport )
#		else
#			define GeneratorAPI __declspec( dllimport )
#		endif
#	endif
#else
#	define GeneratorAPI
#endif

#include <set>
#include <map>
#include <list>
#include <vector>
#include <limits>
#include <algorithm>
#include <iostream>
#include <functional>
#include <cmath>
#include <cstring>
#include <cstdint>

#include "glew.h"

#if defined( _MSC_VER )
#	pragma warning( disable:4251 )	// ... needs to have dll-interface to be used by clients ...
#	pragma warning( disable:4275 )	// ... non dll-interface class 'wxPanel' used as base for dll-interface class ...
#	pragma warning( push )
#	pragma warning( disable:4311 )	// 'cast de type': troncation de pointeur de 'PVOID'  'LONG'	
#	pragma warning( disable:4312 )	// 'cast de type': la conversion de 'LONG' en 'PVOID' d'une taille suprieure
#endif

#include <wx/app.h>
#include <wx/button.h>
#include <wx/choicdlg.h>
#include <wx/colordlg.h>
#include <wx/combobox.h>
#include <wx/dcclient.h>
#include <wx/dcmemory.h>
#include <wx/dialog.h>
#include <wx/display.h>
#include <wx/dynlib.h>
#include <wx/event.h>
#include <wx/filedlg.h>
#include <wx/filefn.h>
#include <wx/filename.h>
#include <wx/fontdlg.h>
#include <wx/frame.h>
#include <wx/glcanvas.h>
#include <wx/icon.h>
#include <wx/image.h>
#include <wx/notebook.h>
#include <wx/numdlg.h>
#include <wx/panel.h>
#include <wx/rawbmp.h>
#include <wx/slider.h>
#include <wx/stattext.h>
#include <wx/statline.h>
#include <wx/statusbr.h>
#include <wx/stdpaths.h>
#include <wx/sysopt.h>
#include <wx/textfile.h>
#include <wx/thread.h>
#include <wx/tglbtn.h>
#include <wx/timer.h>
#include <wx/utils.h>
#include <wx/wx.h>

#ifdef __WXMSW__
#	include<wx/msw/msvcrt.h>
#endif

#if defined( _MSC_VER )
#	pragma warning( pop )
#endif

namespace ProceduralTextures
{
#if defined( __WXMSW__ )
#	define wxDEFAULT_HEIGHT	25
#else
#	define wxDEFAULT_HEIGHT	30
#	define Sleep( x )		usleep( (x) * 1000 )
#endif
	////////////////////////////////////////////////////////////////////////////////////////////////////
	//	GUI
	////////////////////////////////////////////////////////////////////////////////////////////////////
	/*!
	\author		Sylvain DOREMUS
	\date		23/05/2012
	\version 	1.3.0.0
	\~english
	\brief		Enumeration of supported controls in a ConfigPanel
	\~french
	\brief		Enumration des contrles supports par ConfigPanel
	*/
	typedef enum
	{	eCONTROL_TYPE_NONE
	,	eCONTROL_TYPE_STATIC	//!< wxStatic
	,	eCONTROL_TYPE_EDIT		//!< wxTextCtrl
	,	eCONTROL_TYPE_SLIDER	//!< wxSlider
	,	eCONTROL_TYPE_COMBO		//!< wxComboBox
	,	eCONTROL_TYPE_BUTTON	//!< wxButton
	}	eCONTROL_TYPE;
	/*!
	\author		Sylvain DOREMUS
	\date		23/05/2012
	\version 	1.3.0.0
	\~english
	\brief		Enumeration of supported events in ControlInfos
	\~french
	\brief		Enumration des vnements supports par ControlInfos
	*/
	typedef enum
	{	eEVENT_TYPE_NONE
	,	eEVENT_TYPE_SLIDER_UPDATED			//!<	Slider updated
	,	eEVENT_TYPE_SCROLL_THUMBRELEASE		//!<	Scroll thumb release
	,	eEVENT_TYPE_SCROLL_THUMBTRACK		//!<	Scroll thumb track
	,	eEVENT_TYPE_COMBOBOX_SELECTED		//!<	Combo box element selected
	,	eEVENT_TYPE_BUTTON_CLIKED			//!<	Button clicked
	,	eEVENT_TYPE_TEXT_UPDATED			//!<	Keyboard text updated
	,	eEVENT_TYPE_TEXT_ENTER				//!<	Keyboard text return
	,	eEVENT_TYPE_LOAD_TEXT_FILEA			//!<	Button load text file clicked
	,	eEVENT_TYPE_LOAD_TEXT_FILEB			//!<	Button load another text file clicked
	,	eEVENT_TYPE_LOAD_IMAGEA				//!<	Button load image clicked
	,	eEVENT_TYPE_LOAD_IMAGEB				//!<	Button load another image clicked
	,	eEVENT_TYPE_LOAD_BINARY_FILEA		//!<	Button load binary file clicked
	,	eEVENT_TYPE_LOAD_BINARY_FILEB		//!<	Button load another binary file clicked
	,	eEVENT_TYPE_LOAD_COLOURA			//!<	Button load colour clicked
	,	eEVENT_TYPE_LOAD_COLOURB			//!<	Button load another colour clicked
	,	eEVENT_TYPE_COUNT
	}	eEVENT_TYPE;
	/*!
	\author		Sylvain DOREMUS
	\date		23/05/2012
	\version 	1.3.0.0
	\~english
	\brief		Base class for control specific data
	\remark		Objects of this class are uncopyable
	\~french
	\brief		Classe de base pour les donnes spcifiques d'un contrle
	\remark		Les objets de cette classe ne sont pas copyables
	*/
	class SpecificsBase;
	/*!
	\author		Sylvain DOREMUS
	\date		23/05/2012
	\version 	1.3.0.0
	\~english
	\brief		Template class to implement SpecificsBase
	\~french
	\brief		Classe modle pour implmenter SpecificsBase
	*/
	template< eCONTROL_TYPE Type > class Specifics;
	/*!
	\author		Sylvain DOREMUS
	\date		23/05/2012
	\version 	1.3.0.0
	\~english
	\brief		Class describing a control, to allow it's creation by ConfigPanel
	\~french
	\brief		Classe dcrivant un contrle, pour le crer dans ConfigPanel
	*/
	class ControlInfos;
	/*!
	\author		Sylvain DOREMUS
	\date		23/05/2012
	\version 	1.3.0.0
	\~english
	\brief		Panel holding generator tweaks
	\~french
	\brief		Panneau contenant les modifieurs du gnrateur
	*/
	class ConfigPanel;

	////////////////////////////////////////////////////////////////////////////////////////////////////
	//	MAIN
	////////////////////////////////////////////////////////////////////////////////////////////////////
	/*!
	\author		Sylvain DOREMUS
	\date		23/05/2012
	\version 	1.3.0.0
	\~english
	\brief		Enumeration of event ids
	\~french
	\brief		Enumration d'ids d'vnements
	*/
	typedef enum
	{	eEVENT_WORKER_THREAD_END	= 100000	//!<\~english Event launched on worker thread step end					\~french Evnement lanc par un thread de travail pour dire qu'il a fini son travail
	,	eEVENT_THREAD_STOP			= 100001	//!<\~english Event launched to stop main thread 						\~french Evnement lanc pour terminer le thread principal
	,	eEVENT_CPU_STEP_END			= 100002	//!<\~english Event launched by main thread when computation ends		\~french Evnement lanc par le thread principal quand les calculs sont termins
	,	eEVENT_TEXTA				= 100003	//!<\~english Event sent to main thread to give it a text file path		\~french Evnement envoy au thread principal pour lui donner l'adresse d'un fichier texte
	,	eEVENT_TEXTB				= 100004	//!<\~english Event sent to main thread to give it a text file path		\~french Evnement envoy au thread principal pour lui donner l'adresse d'un fichier texte
	,	eEVENT_IMAGEA				= 100005	//!<\~english Event sent to main thread to give it an image file path	\~french Evnement envoy au thread principal pour lui donner l'adresse d'un fichier image
	,	eEVENT_IMAGEB				= 100006	//!<\~english Event sent to main thread to give it an image file path	\~french Evnement envoy au thread principal pour lui donner l'adresse d'un fichier image
	,	eEVENT_BINARYA				= 100007	//!<\~english Event sent to main thread to give it a binary file path	\~french Evnement envoy au thread principal pour lui donner l'adresse d'un fichier binaire
	,	eEVENT_BINARYB				= 100008	//!<\~english Event sent to main thread to give it a binary file path	\~french Evnement envoy au thread principal pour lui donner l'adresse d'un fichier binaire
	,	eEVENT_COLOURA				= 100009	//!<\~english Event sent to main thread to give it a colour				\~french Evnement envoy au thread principal pour lui donner une couleur
	,	eEVENT_COLOURB				= 100010	//!<\~english Event sent to main thread to give it a colour				\~french Evnement envoy au thread principal pour lui donner une couleur
	,	eEVENT_MOUSEMOVE			= 100011	//!<\~english Event sent to main thread to retrieve mouse position		\~french Evnement envoy au thread principal pour lui donner la position de la souris
	}	eEVENT;
	/*!
	\author		Sylvain DOREMUS
	\date		03/01/2011
	\version	1.0.0.0
	\~english
	\brief		Policy for various types
	\remark		Policy created to use various types in template classes such as Point, Matrix, and others.
				<br />Holds  operators, initialisation functions, conversion functions, ...
	\~french
	\brief		Politique pour des types divers
	\remark		Cre pour utiliser dans des classes template telles que Point.
				<br />Contient des oprateurs, fonctions d'initialisation, de conversion, ...
	*/
	template <typename T> class Policy;
	/*!
	\author		Sylvain DOREMUS
	\date		03/01/2011
	\version	1.0.0.0
	\~english
	\brief		Base class for points, used to share functions without taking care of templates
	\~french
	\brief		Classe de points, utilise pour partager des fonctions sans se proccuper des temlates
	*/
	class PointBase;
	/*!
	\author		Sylvain DOREMUS
	\date		03/01/2011
	\version	1.0.0.0
	\~english
	\brief		Templated type, dynamic dimensions point class
	\~french
	\brief		Classe de point  type template et dimensions dynamiques
	*/
	template <typename T> class DynPoint;
	/*!
	\author		Sylvain DOREMUS
	\date		03/01/2011
	\version	1.0.0.0
	\~english
	\brief		Templated type, static dimensions point representation
	\remark		The template type must have a defined Policy
	\~french
	\brief		Classe de point  type template et dimensions statiques
	\remark		Le type template doit avoir une Policy
	*/
	template <typename T, size_t Count> class Point;
	/*!
	\author 	Sylvain DOREMUS
	\date 		03/01/2011
	\version 	1.0.0.0
	\~english
	\brief 		Description of a pixel
	\remark		Four components, Red, Green, Blue and Alpha. Component type is templated
	\~french
	\brief 		Description d'un pixel
	\remark		Quatre composantes : Rouge, Vert, Bleu et Alpha. Le type de donnes de la composante est template.
	*/
	template <typename T> class Pixel;
	/*!
	\author 	Sylvain DOREMUS
	\date 		03/01/2011
	\version 	1.0.0.0
	\~english
	\brief 		Pixel buffer description
	\remark		It has 2 dimensions and is row major.
	\~french
	\brief 		Description d'un buffer de pixels
	\remark		Il a 2 dimensions et est row major.
	*/
	template <typename T> class Buffer;
	/*!
	\author		Sylvain DOREMUS
	\date		01/07/2011
	\version	1.2.0.0
	\~english
	\brief		Plugin base class
	\remark		Under MS Windows, in order to have access to every dynamic wxWidget class,
				<br />we must instantiate a winapp for each dll loaded. So plugin extends wxApp and lauches a dllmain.
				<br />It causes troubles with OpenGL, which functions must be loaded in each main, so OpenGL functions
				<br />are loaded in each procedural generator, instead of being loaded in the main application.
	\~french
	\brief		Classe de base pour les plugins
	\remark		Sous MS-Windows, pour avoir accs  chaque classe dynamique de wxWidgets,
				<br />il faut instantier un wxApp pour chaque DLL charge. Les plugins doivent donc tendre wxApp et lancer un DLLMain.
				<br />Ca pose des problmes avec OpenGL, dont les fonctions doivent tre charges dans chaque main, donc les fonctions OpenGL
				<br />sont charges dans chaque ProceduralGenerator plutt que d'tre charges dans l'application principale.
	*/
	class PluginBase;
	/*!
	\author		Sylvain DOREMUS
	\date		01/07/2011
	\version	1.2.0.0
	\~english
	\brief		Plugin class, to be instantiated in each generator
	\~french
	\brief		Classe de plugin,  instantier dans chaque gnrateur
	*/
	template< class GeneratorClass, class PluginClass > class Plugin;
	/*!
	\author		Sylvain DOREMUS
	\date		01/07/2011
	\version	1.0.0.0
	\~english
	\brief		Random number generator class
	\~french
	\brief		Classe gnration alatoire de nombres
	*/
	class Random;
	/*!
	\author		Sylvain DOREMUS
	\date		23/05/2012
	\version	1.0.0.0
	\~english
	\brief		Base class for all generators
	\~french
	\brief		Classe de base pour tous les gnrateurs
	*/
	class ProceduralGenerator;
	/*!
	\author		Sylvain DOREMUS
	\date		23/05/2012
	\version 	1.3.0.0
	\~english
	\brief		Typedef on a Point with 1 float component
	\~french
	\brief		Typedef sur un Point avec 1 composante en float
	*/
	typedef Point< float, 1 > Point1f;
	/*!
	\author		Sylvain DOREMUS
	\date		23/05/2012
	\version 	1.3.0.0
	\~english
	\brief		Typedef on a Point with 2 float components
	\~french
	\brief		Typedef sur un Point avec 2 composantes en float
	*/
	typedef Point< float, 2 > Point2f;
	/*!
	\author		Sylvain DOREMUS
	\date		23/05/2012
	\version 	1.3.0.0
	\~english
	\brief		Typedef on a Point with 3 float components
	\~french
	\brief		Typedef sur un Point avec 3 composantes en float
	*/
	typedef Point< float, 3 > Point3f;
	/*!
	\author		Sylvain DOREMUS
	\date		23/05/2012
	\version 	1.3.0.0
	\~english
	\brief		Typedef on a Point with 4 float components
	\~french
	\brief		Typedef sur un Point avec 4 composantes en float
	*/
	typedef Point< float, 4 > Point4f;
	/*!
	\author		Sylvain DOREMUS
	\date		23/05/2012
	\version 	1.3.0.0
	\~english
	\brief		Typedef on a Point with 1 int component
	\~french
	\brief		Typedef sur un Point avec 1 composante en int
	*/
	typedef Point< int32_t, 1 > Point1i;
	/*!
	\author		Sylvain DOREMUS
	\date		23/05/2012
	\version 	1.3.0.0
	\~english
	\brief		Typedef on a Point with 2 int components
	\~french
	\brief		Typedef sur un Point avec 2 composantes en int
	*/
	typedef Point< int32_t, 2 > Point2i;
	/*!
	\author		Sylvain DOREMUS
	\date		23/05/2012
	\version 	1.3.0.0
	\~english
	\brief		Typedef on a Point with 3 int components
	\~french
	\brief		Typedef sur un Point avec 3 composantes en int
	*/
	typedef Point< int32_t, 3 > Point3i;
	/*!
	\author		Sylvain DOREMUS
	\date		23/05/2012
	\version 	1.3.0.0
	\~english
	\brief		Typedef on a Point with 4 int components
	\~french
	\brief		Typedef sur un Point avec 4 composantes en int
	*/
	typedef Point< int32_t, 4 > Point4i;
	/*!
	\author		Sylvain DOREMUS
	\date		23/05/2012
	\version 	1.3.0.0
	\~english
	\brief		Typedef on a Point with 1 byte component
	\~french
	\brief		Typedef sur un Point avec 1 composante en byte
	*/
	typedef Point< uint8_t, 1 > Point1ub;
	/*!
	\author		Sylvain DOREMUS
	\date		23/05/2012
	\version 	1.3.0.0
	\~english
	\brief		Typedef on a Point with 2 byte components
	\~french
	\brief		Typedef sur un Point avec 2 composantes en byte
	*/
	typedef Point< uint8_t, 2 > Point2ub;
	/*!
	\author		Sylvain DOREMUS
	\date		23/05/2012
	\version 	1.3.0.0
	\~english
	\brief		Typedef on a Point with 3 byte components
	\~french
	\brief		Typedef sur un Point avec 3 composantes en byte
	*/
	typedef Point< uint8_t, 3 > Point3ub;
	/*!
	\author		Sylvain DOREMUS
	\date		23/05/2012
	\version 	1.3.0.0
	\~english
	\brief		Typedef on a Point with 4 byte components
	\~french
	\brief		Typedef sur un Point avec 4 composantes en byte
	*/
	typedef Point< uint8_t, 4 > Point4ub;
	/*!
	\author		Sylvain DOREMUS
	\date		23/05/2012
	\version 	1.3.0.0
	\~english
	\brief		Typedef on a pixel with byte components
	\~french
	\brief		Typedef sur un pixel avec des composantes en byte
	*/
	typedef Pixel< uint8_t > UbPixel;
	/*!
	\author		Sylvain DOREMUS
	\date		23/05/2012
	\version 	1.3.0.0
	\~english
	\brief		Typedef on a pixel with unsigned int components
	\~french
	\brief		Typedef sur un pixel avec des composantes en unsigned int
	*/
	typedef Pixel< uint32_t > UiPixel;
	/*!
	\author		Sylvain DOREMUS
	\date		23/05/2012
	\version 	1.3.0.0
	\~english
	\brief		Typedef on an array of ControlInfos
	\~french
	\brief		Typedef sur un tableau de ControlInfos
	*/
	typedef Buffer<  UbPixel> PixelBuffer;

	////////////////////////////////////////////////////////////////////////////////////////////////////
	//	GPU
	////////////////////////////////////////////////////////////////////////////////////////////////////
	/*!
	\author 	Sylvain DOREMUS
	\date 		03/01/2011
	\version 	1.0.0.0
	\~english
	\brief		GPU buffer element usage enumerator
	\~french
	\brief		Enumration des utilisations des lments de buffer GPU
	*/
	typedef enum
	{	eELEMENT_USAGE_POSITION		//!< Position vertex
	,	eELEMENT_USAGE_NORMAL		//!< Normal vertex
	,	eELEMENT_USAGE_TANGENT		//!< Tangent vertex
	,	eELEMENT_USAGE_DIFFUSE		//!< Diffuse colour
	,	eELEMENT_USAGE_BITANGENT	//!< Bitangent coords
	,	eELEMENT_USAGE_TEXTURE0		//!< Texture coordinates 0
	,	eELEMENT_USAGE_TEXTURE1		//!< Texture coordinates 1
	,	eELEMENT_USAGE_TEXTURE2		//!< Texture coordinates 2
	,	eELEMENT_USAGE_TEXTURE3		//!< Texture coordinates 3
	,	eELEMENT_USAGE_COUNT
	}	eELEMENT_USAGE;
	/*!
	\author 	Sylvain DOREMUS
	\date 		03/01/2011
	\version 	1.0.0.0
	\~english
	\brief		GPU buffer element data type enumerator
	\~french
	\brief		Enumration des types de donnes d'lments de buffer GPU
	*/
	typedef enum
	{	eELEMENT_TYPE_1FLOAT	//!< 1 float
	,	eELEMENT_TYPE_2FLOATS	//!< 2 floats
	,	eELEMENT_TYPE_3FLOATS	//!< 3 floats
	,	eELEMENT_TYPE_4FLOATS	//!< 4 floats
	,	eELEMENT_TYPE_COLOUR	//!< colour (unsigned int)
	,	eELEMENT_TYPE_COUNT
	}	eELEMENT_TYPE;
	/*!
	\author 	Sylvain DOREMUS
	\date		01/07/2011
	\version 	1.2.0.0
	\~english
	\brief		Shader object supported types enumerator
	\~french
	\brief		Enumration des types de shaders supports
	*/
	typedef enum
	{	eSHADER_OBJECT_TYPE_NONE		= -1
	,	eSHADER_OBJECT_TYPE_VERTEX		= 0		//!< Vertex shader
	,	eSHADER_OBJECT_TYPE_PIXEL		= 1		//!< Fragment shader
	,	eSHADER_OBJECT_TYPE_GEOMETRY	= 2		//!< Geometry shader
	,	eSHADER_OBJECT_TYPE_HULL		= 3		//!< Tessellation control shader
	,	eSHADER_OBJECT_TYPE_DOMAIN		= 4		//!< Tessellation evaluator shader
	,	eSHADER_OBJECT_TYPE_COUNT
	}	eSHADER_OBJECT_TYPE;
	/*!
	\author 	Sylvain DOREMUS
	\date 		03/01/2011
	\version 	1.0.0.0
	\~english
	\brief 		GPU buffer component description
	\remark		Describes usage and type of one component of a GPU buffer
	\~french
	\brief 		Description d'une composante de buffer GPU
	\remark		Dcrit l'utilisation et le type d'une composante d'un buffer GPU
	*/
	struct BufferElementDeclaration;
	/*!
	\author 	Sylvain DOREMUS
	\date 		03/01/2011
	\version 	1.0.0.0
	\~english
	\brief 		GPU buffer components declaration
	\remark		Describes all components of a GPU buffer
	\~french
	\brief 		Dclaration des composantes d'un buffer GPU
	\remark		Dcrit toutes les composantes d'un buffer GPU
	*/
	class BufferDeclaration;
	/*!
	\author 	Sylvain DOREMUS
	\date 		03/01/2011
	\version 	1.0.0.0
	\~english
	\brief 		Definition of an element in a GPU buffer
	\~french
	\brief 		Dfinition d'un lment dans un buffer GPU
	*/
	class BufferElement;
	/*!
	\author		Sylvain DOREMUS
	\date 		03/01/2011
	\version 	1.0.0.0
	\~english
	\brief		Representation of a vertex
	\remark		Specialisation of BufferElement, containing vertex's position, normal, tangent and texture coordinates
	\~french
	\brief		Reprsentation d'un vertex
	\remark		Spcialisation de BufferElement, contient les coordonnes du vertex, sa normale, sa tangente et ses coordonnes de texture
	*/
	class Vertex;
	/*!
	\author		Sylvain DOREMUS
	\date 		03/01/2011
	\version 	1.0.0.0
	\~english
	\brief		OpenGL GPU buffer base class
	\~french
	\brief		Classe de base pour les buffers GPU OpenGL
	*/
	class GlBuffer;
	/*!
	\author		Sylvain DOREMUS
	\date 		03/01/2011
	\version 	1.0.0.0
	\~english
	\brief		OpenGL Vertex Buffer Object
	\~french
	\brief		Vertex Buffer Object OpenGL
	*/
	class GlVertexBuffer;
	/*!
	\author		Sylvain DOREMUS
	\date 		03/01/2011
	\version 	1.0.0.0
	\~english
	\brief		OpenGL Index Buffer Object
	\~french
	\brief		Index Buffer Object OpenGL
	*/
	class GlIndexBuffer;
	/*!
	\author		Sylvain DOREMUS
	\date		01/07/2011
	\version 	1.2.0.0
	\~english
	\brief		OpenGL Pixel Buffer Object base class
	\~french
	\brief		Classe de base pour les Pixel Buffer Objects OpenGL
	*/
	class GlPixelBuffer;
	/*!
	\author		Sylvain DOREMUS
	\date		01/07/2011
	\version 	1.2.0.0
	\~english
	\brief		Pixel Buffer Object used to send data to GPU
	\~french
	\brief		Pixel Buffer Object utilis pour envoyer les donnes au GPU
	*/
	class GlPackPixelBuffer;
	/*!
	\author		Sylvain DOREMUS
	\date		01/07/2011
	\version 	1.2.0.0
	\~english
	\brief		Pixel Buffer Object used to retrieve data from GPU
	\~french
	\brief		Pixel Buffer Object utilis pour rcuprer les donne du GPU
	*/
	class GlUnpackPixelBuffer;
	/*!
	\author		Sylvain DOREMUS
	\date 		03/01/2011
	\version 	1.0.0.0
	\~english
	\brief		OpenGL texture representation
	\remark		Uses 2 Pixel Buffer Objects to transfer data from/to GPU to/from RAM
	\~french
	\brief		Reprsentation d'une texture OpenGL
	\remark		Utilise 2 Pixel Buffer Objects pour transfrer les donnes du (vers le) GPU vers ( partir de) la RAM
	*/
	class GlTexture;
	/*!
	\author		Sylvain DOREMUS
	\date		01/07/2011
	\version 	1.2.0.0
	\~english
	\brief		OpenGL Frame Buffer Object
	\~french
	\brief		Frame Buffer Object OpenGL
	*/
	class GlFrameBuffer;
	/*!
	\author		Sylvain DOREMUS
	\date		01/07/2011
	\version 	1.2.0.0
	\~english
	\brief		Frame variable (uniforms) base class
	\~french
	\brief		Classe de base pour les variable de frame (uniforms)
	*/
	class GlFrameVariableBase;
	/*!
	\author		Sylvain DOREMUS
	\date		01/07/2011
	\version 	1.2.0.0
	\~english
	\brief		Type templated frame variable class
	\~french
	\brief		Classe modle pour les variable de frame
	*/
	template <typename T> class GlFrameVariable;
	/*!
	\author		Sylvain DOREMUS
	\date		01/07/2011
	\version 	1.2.0.0
	\~english
	\brief		Shader object class (vertex, fragment, ...)
	\~french
	\brief		Classe d'un objet shader (vertex, fragment, ...)
	\remark
	*/
	class GlShaderObject;
	/*!
	\author		Sylvain DOREMUS
	\date		01/07/2011
	\version 	1.2.0.0
	\~english
	\brief		Shader program class, holds shader objects
	\~french
	\brief		Classe de programme de shader, contient les objets de shader
	*/
	class GlShaderProgram;
	/*!
	\author		Sylvain DOREMUS
	\date		01/07/2011
	\version 	1.2.0.0
	\~english
	\brief		Typedef on a list of GlFrameVariableBase
	\~french
	\brief		Typedef sur une liste de GlFrameVariableBase
	*/
	typedef std::list<GlFrameVariableBase *> FrameVariablePtrList;
	/*!
	\author		Sylvain DOREMUS
	\date		01/07/2011
	\version 	1.2.0.0
	\~english
	\brief		Typedef on a map of GlFrameVariableBase, ordered by name
	\~french
	\brief		Typedef sur une map de GlFrameVariableBase, tries par nom
	*/
	typedef std::map<wxString, GlFrameVariableBase *> FrameVariablePtrStrMap;

	namespace detail
	{
		/**
		 *\~english
		 *\brief		Counts available CPU
		 *\return		The CPU count
		 *\~french
		 *\brief		Compte le nombre de CPU disponibles
		 *\return		Le compte des CPU
		 */
		int GetCPUCount();
	}
}

#endif
