/*
This source file is part of ProceduralGenerator (https://sourceforge.net/projects/proceduralgene/)

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU Lesser General Public License as published by the Free Software
Foundation; either version 2 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License along with
the program; if not, write to the Free Software Foundation, Inc., 59 Temple
Place - Suite 330, Boston, MA 02111-1307, USA, or go to
http://www.gnu.org/copyleft/lesser.txt.
*/
#ifndef ___ImageEffects___
#define ___ImageEffects___

#include <Generator/ConfigPanel.h>

namespace ProceduralTextures
{
	class ImageEffects : public ProceduralGenerator
	{
	public:
		typedef enum
		{	eEFFECT_TYPE_NONE
		,	eEFFECT_TYPE_GAUSSIAN_BLUR
		,	eEFFECT_TYPE_SHARPEN
		,	eEFFECT_TYPE_GAUSSIAN_BLUR_SHARPEN
		,	eEFFECT_TYPE_LINEAR_NR
		,	eEFFECT_TYPE_ANISOTROPIC_NR
		,	eEFFECT_TYPE_WAVE
		,	eEFFECT_TYPE_GRAYSCALE
		,	eEFFECT_TYPE_SEPIA
		,	eEFFECT_TYPE_IMPLOSION
		,	eEFFECT_TYPE_EXPLOSION
		,	eEFFECT_TYPE_WATERSHED
		,	eEFFECT_TYPE_REC_GAUSSIAN_BLUR
		,	eEFFECT_TYPE_REC_SHARPEN
		,	eEFFECT_TYPE_REC_GAUSSIAN_BLUR_SHARPEN
		,	eEFFECT_TYPE_REC_LINEAR_NR
		,	eEFFECT_TYPE_REC_ANISOTROPIC_NR
		,	eEFFECT_TYPE_COUNT
		}	eEFFECT_TYPE;

	public:
		class EffectFunctor
		{
		protected:
			eEFFECT_TYPE		m_eEffectType;
			bool				m_bAllowsSecondEffect;
			bool				m_bCanBeSecondEffect;
			int					m_iImgWidth;
			int					m_iImgHeight;
			wxString			m_strName;
			ImageEffects	*	m_pGenerator;

		public:
			EffectFunctor( eEFFECT_TYPE	p_eEffectType, const wxString & m_strName, bool p_bAllowsSecondEffect, bool p_bCanBeSecondEffect);
			virtual ~EffectFunctor();

			virtual void operator ()( const PixelBuffer & p_bufferIn, PixelBuffer & p_bufferOut)=0;

			virtual void SetImage( const wxImage & p_image);

		public:
			inline bool				IsAllowsSecondEffect	()const { return m_bAllowsSecondEffect; }
			inline bool				IsCanBeSecondEffect		()const { return m_bCanBeSecondEffect; }
			inline eEFFECT_TYPE		GetEffectType			()const { return m_eEffectType; }
			inline const wxString &	GetName					()const	{ return m_strName; }

			inline	void	SetImgWidth		( int p_iVal)					{ m_iImgWidth = p_iVal; }
			inline	void	SetImgHeight	( int p_iVal)					{ m_iImgHeight = p_iVal; }
			inline	void	SetGenerator	( ImageEffects * p_pGenerator)	{ m_pGenerator = p_pGenerator; }
		};

	private:
		class NoneFunctor : public EffectFunctor
		{
		public:
			NoneFunctor();
			virtual ~NoneFunctor();

			virtual void operator ()( const PixelBuffer & p_bufferIn, PixelBuffer & p_bufferOut);
		};

		class GaussianBlurFunctor : public EffectFunctor
		{
		private:
			static const int		m_iGaussLvl;
			static const int		m_iGaussMask[];
			static const int		m_iGaussSum;
			PixelBuffer				m_temporaryBuffer;

		public:
			GaussianBlurFunctor();
			virtual ~GaussianBlurFunctor();

			virtual void operator ()( const PixelBuffer & p_bufferIn, PixelBuffer & p_bufferOut);

			virtual void SetImage( const wxImage & p_image);
		};

		class SharpenFunctor : public EffectFunctor
		{
		private:
			static const double		m_dSharpenZ;
			static const double		m_dSharpenMask[];

		public:
			SharpenFunctor();
			virtual ~SharpenFunctor();

			virtual void operator ()( const PixelBuffer & p_bufferIn, PixelBuffer & p_bufferOut);
		};

		class GaussianBlurSharpenFunctor : public EffectFunctor
		{
		private:
			static const int		m_iGaussLvl;
			static const int		m_iGaussMask[];
			static const int		m_iGaussSum;
			static const double		m_dSharpenZ;
			static const double		m_dSharpenMask[];
			PixelBuffer				m_temporaryBuffer1;
			PixelBuffer				m_temporaryBuffer2;

		public:
			GaussianBlurSharpenFunctor();
			virtual ~GaussianBlurSharpenFunctor();

			virtual void operator ()( const PixelBuffer & p_bufferIn, PixelBuffer & p_bufferOut);

			virtual void SetImage( const wxImage & p_image);
		};

		class LinearNoiseReductionFunctor : public EffectFunctor
		{
		private:
			static const int		m_iLnrMask[];

		public:
			LinearNoiseReductionFunctor();
			virtual ~LinearNoiseReductionFunctor();

			virtual void operator ()( const PixelBuffer & p_bufferIn, PixelBuffer & p_bufferOut);
		};

		class AnisotropicNoiseReductionFunctor : public EffectFunctor
		{
		private:
			static const int		m_iAdThreshold;
			static const int		m_iAdMask[];
			static const int		m_iAdSum;

		public:
			AnisotropicNoiseReductionFunctor();
			virtual ~AnisotropicNoiseReductionFunctor();

			virtual void operator ()( const PixelBuffer & p_bufferIn, PixelBuffer & p_bufferOut);
		};

		class WaveFunctor : public EffectFunctor
		{
		private:
			static double	m_dDegToRad;
			static double	m_dPiMult2;
			static double	m_dCos[360];
			static double	m_dSin[360];
			double	m_dTimeIndexX;
			double	m_dTimeIndexY;
			int		m_iDecalX;
			int		m_iDecalY;
			double	m_dTimeScaleX;
			double	m_dTimeScaleY;
			double	m_dScaleX;
			double	m_dScaleY;
			double	m_dTranslateX;
			double	m_dTranslateY;
			double	m_dCurTranslateX;
			double	m_dCurTranslateY;
			double	m_dNbPixelsX;
			double	m_dNbPixelsY;

		public:
			WaveFunctor();
			virtual ~WaveFunctor();

			virtual void operator ()( const PixelBuffer & p_bufferIn, PixelBuffer & p_bufferOut);
		};

		class GrayScaleFunctor : public EffectFunctor
		{
		public:
			GrayScaleFunctor();
			virtual ~GrayScaleFunctor();

			virtual void operator ()( const PixelBuffer & p_bufferIn, PixelBuffer & p_bufferOut);
		};

		class SepiaFunctor : public EffectFunctor
		{
		public:
			SepiaFunctor();
			virtual ~SepiaFunctor();

			virtual void operator ()( const PixelBuffer & p_bufferIn, PixelBuffer & p_bufferOut);
		};

		class ImplosionFunctor : public EffectFunctor
		{
		public:
			ImplosionFunctor();
			virtual ~ImplosionFunctor();

			virtual void operator ()( const PixelBuffer & p_bufferIn, PixelBuffer & p_bufferOut);
		};

		class ExplosionFunctor : public EffectFunctor
		{
		public:
			ExplosionFunctor();
			virtual ~ExplosionFunctor();

			virtual void operator ()( const PixelBuffer & p_bufferIn, PixelBuffer & p_bufferOut);
		};

		class WatershedFunctor : public EffectFunctor
		{
		private:
			int m_iNbSteps;

		public:
			WatershedFunctor();
			virtual ~WatershedFunctor();

			virtual void operator ()( const PixelBuffer & p_bufferIn, PixelBuffer & p_bufferOut);
		};

		class RecursiveGaussianBlurFunctor : public EffectFunctor
		{
		private:
			GaussianBlurFunctor m_onceFunctor;

		public:
			RecursiveGaussianBlurFunctor();
			virtual ~RecursiveGaussianBlurFunctor();

			virtual void operator ()( const PixelBuffer & p_bufferIn, PixelBuffer & p_bufferOut);

			virtual void	SetImage( const wxImage & p_image);
		};

		class RecursiveSharpenFunctor : public EffectFunctor
		{
		private:
			SharpenFunctor m_onceFunctor;

		public:
			RecursiveSharpenFunctor();
			virtual ~RecursiveSharpenFunctor();

			virtual void operator ()( const PixelBuffer & p_bufferIn, PixelBuffer & p_bufferOut);

			virtual void	SetImage( const wxImage & p_image);
		};

		class RecursiveGaussianBlurSharpenFunctor : public EffectFunctor
		{
		private:
			GaussianBlurSharpenFunctor	m_onceFunctor;

		public:
			RecursiveGaussianBlurSharpenFunctor();
			virtual ~RecursiveGaussianBlurSharpenFunctor();

			virtual void operator ()( const PixelBuffer & p_bufferIn, PixelBuffer & p_bufferOut);

			virtual void	SetImage( const wxImage & p_image);
		};

		class RecursiveLinearNoiseReductionFunctor : public EffectFunctor
		{
		private:
			LinearNoiseReductionFunctor m_onceFunctor;

		public:
			RecursiveLinearNoiseReductionFunctor();
			virtual ~RecursiveLinearNoiseReductionFunctor();

			virtual void operator ()( const PixelBuffer & p_bufferIn, PixelBuffer & p_bufferOut);

			virtual void	SetImage( const wxImage & p_image);
		};

		class RecursiveAnisotropicNoiseReductionFunctor : public EffectFunctor
		{
		private:
			AnisotropicNoiseReductionFunctor m_onceFunctor;

		public:
			RecursiveAnisotropicNoiseReductionFunctor();
			virtual ~RecursiveAnisotropicNoiseReductionFunctor();

			virtual void operator ()( const PixelBuffer & p_bufferIn, PixelBuffer & p_bufferOut);

			virtual void	SetImage( const wxImage & p_image);
		};

		class Thread : public ProceduralGenerator::Thread
		{
		private:
			int							m_iImgWidth;
			int							m_iImgHeight;
			EffectFunctor			*	m_pFirstFunction;
			EffectFunctor			*	m_pNextFirstFunction;
			EffectFunctor			*	m_pSecondFunction;
			EffectFunctor			*	m_pNextSecondFunction;
			PixelBuffer				*	m_pFirstFunctionBufferIn;
			PixelBuffer				*	m_pNextFirstFunctionBufferIn;

		public:
			PixelBuffer				*	m_backBuffer;
			PixelBuffer					m_firstInBuffer;
			PixelBuffer					m_secondInBuffer;
			static EffectFunctor	*	m_pFunctors[eEFFECT_TYPE_COUNT];

		public:
			Thread( ProceduralGenerator * p_pParent, size_t p_uiIndex, int iWidth, int iTop, int iBottom, int iTotalHeight, const UbPixel & p_pxColour);
			virtual ~Thread();

			virtual void Step();

			void SetImage( const wxImage & p_image);
			void SetFirstFunction( eEFFECT_TYPE p_eType);
			void SetSecondFunction( eEFFECT_TYPE p_eType);

			static void InitialiseFunctors( ImageEffects * p_pParent);
			static void CleanupFunctors();
		};

	private:
		wxImage											m_image;
		Specifics< eCONTROL_TYPE_STATIC >	m_specStaticFirstFunction;
		Specifics< eCONTROL_TYPE_COMBO >	m_specComboFirstFunction;
		Specifics< eCONTROL_TYPE_STATIC >	m_specStaticSecondFunction;
		Specifics< eCONTROL_TYPE_COMBO >	m_specComboSecondFunction;
		Specifics< eCONTROL_TYPE_BUTTON >	m_specButtonImage;

	public:
		ImageEffects( int p_width, int p_height, int p_iWndId, wxFrame * p_pFrame);
		virtual ~ImageEffects();


		void SetFirstFunction( eEFFECT_TYPE p_eType);
		void SetSecondFunction( eEFFECT_TYPE p_eType);
		void SetImagePath( const wxString & p_strImagePath);

		static EffectFunctor * GetFunctor( eEFFECT_TYPE p_eType);

	private:
		virtual void DoGlInitialise		(){}
		virtual void DoGlPreRender		(){}
		virtual void DoGlRender			( bool & WXUNUSED( p_bChanged ) ){}
		virtual void DoGlPostRender		(){}
		virtual void DoGlCleanup		(){}
		virtual void DoGeneratePanel	();

		typedef enum
		{	eID_FIRSTFUNCTION	= 50
		,	eID_SECONDFUNCTION	= 51
		,	eID_IMAGEPATH		= 52
		}	eID;

		void OnSize				( wxSizeEvent		&	p_event );
		void OnFirstFunction	( wxCommandEvent	&	p_event );
		void OnSecondFunction	( wxCommandEvent	&	p_event );
		void OnImagePath		( wxCommandEvent	&	p_event );
	};

	class IePlugin : public Plugin< ImageEffects, IePlugin >
	{
	public:
		IePlugin();
		virtual ~IePlugin();

		virtual wxString GetName();

		DECLARE_PLUGIN()
	};
}

#endif
