/*
This source file is part of ProceduralGenerator (https://sourceforge.net/projects/proceduralgene/)

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU Lesser General Public License as published by the Free Software
Foundation; either version 2 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License along with
the program; if not, write to the Free Software Foundation, Inc., 59 Temple
Place - Suite 330, Boston, MA 02111-1307, USA, or go to
http://www.gnu.org/copyleft/lesser.txt.
*/
#ifndef ___ShaderWebcam___
#define ___ShaderWebcam___

#if defined( _MSC_VER )
#	pragma warning( push )
#	pragma warning( disable:4311 )	// 'cast de type': troncation de pointeur de 'PVOID'  'LONG'	
#	pragma warning( disable:4312 )	// 'cast de type': la conversion de 'LONG' en 'PVOID' d'une taille suprieure
#	pragma warning( disable:4267 )	// 'argument': conversion de 'size_t' en 'unsigned int', perte possible de donnes
#endif

#include <Generator/ConfigPanel.h>

#if defined( new )
#	undef new
#endif

#include <opencv/cv.h>
#include <opencv/highgui.h>

namespace ProceduralTextures
{
	class SwEffectsNotebook;
	class SwEffectPage;
	class ShaderWebcam;
	class SwPlugin;
	class SwPanel;

	typedef enum
	{	eSEPARATION_NONE
	,	eSEPARATION_HORIZ
	,	eSEPARATION_VERT
	}	eSEPARATION;


	class Effect
	{
	private:
		wxImage *					m_pImage;
		std::vector <wxImage *>		m_arrayImages;
		std::vector <wxString>		m_arrayImagesFiles;
		GlShaderProgram			*	m_pShaderProgram;
		wxString					m_strVertexFile;
		wxString					m_strFragmentFile;
		bool						m_bNewShader;
		wxMilliClock_t				m_llStartTime;
		int							m_iWidth;
		int							m_iHeight;
		OpenGl *					m_pOpenGl;

	public:
		Effect( OpenGl * p_pOpenGl, wxImage * p_pImage);
		virtual ~Effect();

		void SetVertexFile( const wxString & p_strPath);
		void SetFragmentFile( const wxString & p_strPath);
		void Compile();
		void SetImagePath( size_t p_iImage, const wxString & p_strImagePath);

		void Initialise();
		void Cleanup();

		bool Activate( eSEPARATION p_eSeparationType, int p_iSeparationOffset);
		void Deactivate();

		wxString GetCompilerLog( eSHADER_OBJECT_TYPE p_eType);
		wxString GetLinkerLog();

		void UpdateImages();

		inline void ResetTime() { m_llStartTime = wxGetLocalTimeMillis(); }
		inline bool IsInitialised()const { return !m_bNewShader; }
	};

	class ShaderWebcam : public ProceduralGenerator
	{
	private:
		class Thread : public ProceduralGenerator::Thread
		{
		private:
			IplImage *		m_pImage;
			CvCapture *		m_pCapture;
			PixelBuffer * 	m_pBuffer;

		public:
			Thread( ProceduralGenerator * p_pParent, size_t p_uiIndex, int iWidth, int iTop, int iBottom, int iTotalHeight, const UbPixel & p_pxColour );
			virtual ~Thread();

			void Initialise( CvCapture * p_pCapture, PixelBuffer * p_pBuffer );
			virtual void Step();
		};

	private:
		typedef std::vector< Effect* >	EffectPtrArray;

		CvCapture *							m_pCapture;
		wxImage								m_image;
		EffectPtrArray						m_arrayEffects;
		eSEPARATION							m_eSeparationType;
		int									m_iSeparationOffset;
		GlFrameBuffer						m_frameBuffer1;
		GlFrameBuffer						m_frameBuffer2;
		GlTexture							m_texture1;
		GlTexture							m_texture2;
		Specifics< eCONTROL_TYPE_STATIC >	m_specStaticSeparator;
		Specifics< eCONTROL_TYPE_COMBO >	m_specComboSeparator;
		Specifics< eCONTROL_TYPE_SLIDER >	m_specSliderOffset;
		Specifics< eCONTROL_TYPE_BUTTON >	m_specButtonReset;
		Specifics< eCONTROL_TYPE_STATIC >	m_specStaticShaders;
		Specifics< eCONTROL_TYPE_COMBO >	m_specComboShaders;
		Specifics< eCONTROL_TYPE_BUTTON >	m_specButtonVertexFile;
		Specifics< eCONTROL_TYPE_BUTTON >	m_specButtonFragmentFile;
		Specifics< eCONTROL_TYPE_BUTTON >	m_specButtonCompile;
		Specifics< eCONTROL_TYPE_BUTTON >	m_specButtonCompilerLog;
		Specifics< eCONTROL_TYPE_BUTTON >	m_specButtonRemove;
		Effect *										m_pSelectedEffect;

	public:
		ShaderWebcam( int p_width, int p_height, int p_iWndId, wxFrame * p_pFrame);
		virtual ~ShaderWebcam();

		void ResetTime		();

		bool		EffectAdd				();
		bool		EffectRemove			( size_t p_uiIndex);
		void		EffectSetVertexFile		( size_t p_uiIndex, const wxString & p_strPath);
		void		EffectSetFragmentFile	( size_t p_uiIndex, const wxString & p_strPath);
		void		EffectSetImagePath		( size_t p_uiIndex, size_t p_iImage, const wxString & p_strImagePath);
		void		EffectCompile			( size_t p_uiIndex);
		wxString	EffectGetCompilerLog	( size_t p_uiIndex, eSHADER_OBJECT_TYPE p_eType);
		wxString	EffectGetLinkerLog		( size_t p_uiIndex);
		bool		EffectActivate			( size_t p_uiIndex);
		void		EffectDeactivate		( size_t p_uiIndex);

		inline void SetSeparationOffset	( int p_iOffset)		{ m_iSeparationOffset = p_iOffset; }
		inline void SetSeparationType	( eSEPARATION p_eType)	{ m_eSeparationType = p_eType; }

	private:
		virtual void DoResize			( const wxSize & p_size);
		virtual void DoGlInitialise		();
		virtual void DoGlPreRender		();
		virtual void DoGlRender			( bool & p_bChanged);
		virtual void DoGlPostRender		();
		virtual void DoGlCleanup		();
		virtual void DoGeneratePanel	();

		typedef enum
		{	eID_SEPTYPE			= 53
		,	eID_SEPOFFSET		= 54
		,	eID_RESETTIME		= 55
		,	eID_SHADERS			= 56
		,	eID_REMOVE			= 57
		,	eID_VERTEXFILE		= 58
		,	eID_FRAGMENTFILE	= 59
		,	eID_COMPILE			= 60
		,	eID_COMPILERLOG		= 61
		}	eID;

		void OnSepType				( wxCommandEvent	&	p_event );
		void OnSepOffset			( wxCommandEvent	&	p_event );
		void OnResetTime			( wxCommandEvent	&	p_event );
		void OnSelectShader			( wxCommandEvent	&	p_event );
		void OnRemove				( wxCommandEvent	&	p_event );
		void OnShaderCompile		( wxCommandEvent	&	p_event );
		void OnCompilerLog			( wxCommandEvent	&	p_event );
		void OnVertexShaderPath		( wxCommandEvent	&	p_event );
		void OnFragmentShaderPath	( wxCommandEvent	&	p_event );
	};

	class SwPlugin : public Plugin< ShaderWebcam, SwPlugin >
	{
	public:
		SwPlugin();
		virtual ~SwPlugin();

		virtual wxString GetName();

		DECLARE_PLUGIN()
	};
}

#if defined( _MSC_VER )
#	pragma warning( pop )
#endif

#endif
