#include "Generator/PrecompiledHeader.h"

#include "Generator/ProceduralGenerator.h"
#include "Generator/GlObjects.h"

using namespace ProceduralTextures;

//*************************************************************************************************

#ifdef _WIN32
#	define LibError() GetLastError()
#else
#	define LibError() dlerror()
#endif

//*************************************************************************************************

TextOverlay::Glyph :: Glyph()
{
}

TextOverlay::Glyph :: Glyph( const Glyph & p_glyph )
	:	m_ptPosition	( p_glyph.m_ptPosition	)
	,	m_size			( p_glyph.m_size		)
	,	m_bitmap		( p_glyph.m_bitmap		)
{
}

TextOverlay::Glyph & TextOverlay::Glyph :: operator =( Glyph const & p_glyph )
{
	m_ptPosition	= p_glyph.m_ptPosition;
	m_size			= p_glyph.m_size;
	m_bitmap		= p_glyph.m_bitmap;

	return * this;
}

TextOverlay::Glyph :: ~Glyph()
{
	m_bitmap.clear();
}

//*************************************************************************************************

TextOverlay :: TextOverlay( OpenGl * p_pOpenGl )
	:	m_pOpenGl					( p_pOpenGl	)
	,	m_vtxBuffer					( p_pOpenGl	)
	,	m_idxBuffer					( p_pOpenGl	)
	,	m_textureBackground			( p_pOpenGl	)
	,	m_textureForeground			( p_pOpenGl	)
	,	m_textureText				( p_pOpenGl	)
	,	m_textureBlendedText		( p_pOpenGl	)
	,	m_frameBufferBlendedText	( p_pOpenGl	)
	,	m_textureBackgroundCut		( p_pOpenGl	)
	,	m_frameBufferBackgroundCut	( p_pOpenGl	)
	,	m_textureResult				( p_pOpenGl	)
	,	m_frameBufferResult			( p_pOpenGl	)
	,	m_bTextModified				( false		)
	,	m_bBackgroundModified		( false		)
	,	m_bForegroundModified		( false		)
	,	m_bMatrixModified			( false		)
	,	m_iMaxHeight				( 0			)
	,	m_arrayGlyphs				( 256		)
{
	std::memset( m_matrix, 0, sizeof( m_matrix ) );
	m_matrix[ 0] = 1.0;
	m_matrix[ 5] = 1.0;
	m_matrix[10] = 1.0;
	m_matrix[15] = 1.0;

	m_blendRgbTxt.function		= GL_MODULATE;
	m_blendRgbTxt.source1		= GL_TEXTURE;
	m_blendRgbTxt.operand1		= GL_SRC_ALPHA;
	m_blendRgbTxt.source2		= GL_CONSTANT;
	m_blendRgbTxt.operand2		= GL_SRC_COLOR;
	m_blendRgbTxt.source3		= GL_NONE;
	m_blendRgbTxt.operand3		= GL_NONE;

	m_blendAlphaTxt.function	= GL_REPLACE;
	m_blendAlphaTxt.source1		= GL_TEXTURE;
	m_blendAlphaTxt.operand1	= GL_SRC_ALPHA;
	m_blendAlphaTxt.source2		= GL_NONE;
	m_blendAlphaTxt.operand2	= GL_NONE;
	m_blendAlphaTxt.source3		= GL_NONE;
	m_blendAlphaTxt.operand3	= GL_NONE;

	m_blendRgbImg.function		= GL_MODULATE;
	m_blendRgbImg.source1		= GL_PREVIOUS;
	m_blendRgbImg.operand1		= GL_ONE_MINUS_SRC_ALPHA;
	m_blendRgbImg.source2		= GL_TEXTURE;
	m_blendRgbImg.operand2		= GL_SRC_COLOR;
	m_blendRgbImg.source3		= GL_NONE;
	m_blendRgbImg.operand3		= GL_NONE;

	m_blendAlphaImg.function	= GL_REPLACE;
	m_blendAlphaImg.source1		= GL_PREVIOUS;
	m_blendAlphaImg.operand1	= GL_ONE_MINUS_SRC_ALPHA;
	m_blendAlphaImg.source2		= GL_NONE;
	m_blendAlphaImg.operand2	= GL_NONE;
	m_blendAlphaImg.source3		= GL_NONE;
	m_blendAlphaImg.operand3	= GL_NONE;
}

TextOverlay :: ~TextOverlay()
{
}

void TextOverlay :: Initialise()
{
	m_vtxBuffer.Initialise();
	m_idxBuffer.Initialise();

	m_textureBackground.Initialise();
	m_textureForeground.Initialise();
	m_textureText.Initialise();

	m_textureBackgroundCut.Initialise();
	m_frameBufferBackgroundCut.Initialise();

	m_textureBlendedText.Initialise();
	m_frameBufferBlendedText.Initialise();

	m_textureResult.Initialise();
	m_frameBufferResult.Initialise();

	m_textureBackground.Initialise( m_size );
	m_textureForeground.Initialise( m_size );
	m_textureText.Initialise( m_size );

	m_textureBackgroundCut.Initialise( m_size );
	m_frameBufferBackgroundCut.Initialise( m_size );

	m_textureBlendedText.Initialise( m_size );
	m_frameBufferBlendedText.Initialise( m_size );

	m_textureResult.Initialise( m_size );
	m_frameBufferResult.Initialise( m_size );

	m_textureBackground.GetBuffer().init();
	m_textureForeground.GetBuffer().init();
	m_textureText.GetBuffer().init();

	for( int y = 0 ; y < m_size.y ; y++ )
	{
		for( int x = 0 ; x < m_size.x ; x++ )
		{
			m_textureBackground.GetBuffer()[y][x].r = 0;
			m_textureBackground.GetBuffer()[y][x].g = 0;
			m_textureBackground.GetBuffer()[y][x].b = 0;
			m_textureBackground.GetBuffer()[y][x].a = 0;

			m_textureForeground.GetBuffer()[y][x].r = 255;
			m_textureForeground.GetBuffer()[y][x].g = 255;
			m_textureForeground.GetBuffer()[y][x].b = 255;
			m_textureForeground.GetBuffer()[y][x].a = 255;
		}
	}
}

void TextOverlay :: Cleanup()
{
	m_vtxBuffer.Cleanup();
	m_idxBuffer.Cleanup();

	m_textureBackground.Cleanup();
	m_textureForeground.Cleanup();
	m_textureText.Cleanup();

	m_textureBackgroundCut.Cleanup();
	m_frameBufferBackgroundCut.Cleanup();

	m_textureBlendedText.Cleanup();
	m_frameBufferBlendedText.Cleanup();

	m_textureResult.Cleanup();
	m_frameBufferResult.Cleanup();
}

void TextOverlay :: Create( int p_iHeight, wxFontStyle p_eStyle, wxFontWeight p_eWeight )
{
	m_textureBackground.Resize( m_size );
	m_textureForeground.Resize( m_size );
	m_textureText.Resize( m_size );

	m_textureBackgroundCut.Resize( m_size );
	m_frameBufferBackgroundCut.Resize( m_size );
	m_frameBufferBackgroundCut.BindTexture( GL_COLOR_ATTACHMENT0, &m_textureBackgroundCut );

	m_textureBlendedText.Resize( m_size );
	m_frameBufferBlendedText.Resize( m_size );
	m_frameBufferBlendedText.BindTexture( GL_COLOR_ATTACHMENT0, &m_textureBlendedText );

	m_textureResult.Resize( m_size );
	m_frameBufferResult.Resize( m_size );
	m_frameBufferResult.BindTexture( GL_COLOR_ATTACHMENT0, &m_textureResult );

	m_iMaxHeight = p_iHeight + 10;
	wxSize l_sizeImg( m_iMaxHeight, m_iMaxHeight );
	wxFont l_font( wxSize( 0, p_iHeight ), wxFONTFAMILY_SWISS, p_eStyle, p_eWeight, false, wxT( "Arial" ) );
	// We first create an image to write each character into.
	wxBitmap l_bitmapMeasure( l_sizeImg, 24 );
	wxMemoryDC l_dcMeasure( l_bitmapMeasure );
	l_dcMeasure.SetFont( l_font );

	// Then we draw the characters.
	for( uint16_t i = 0 ; i < 256 ; i++ )
	{
		wxChar l_chars[2] = { uint8_t( i ), 0 };
		int l_iWidth = l_dcMeasure.GetTextExtent( l_chars ).x;

		if( l_iWidth > 0 )
		{
			wxBitmap l_bitmap( wxSize( l_iWidth, l_sizeImg.y ), 24 );
			wxMemoryDC l_dc( l_bitmap );

			l_dc.SetBackground( *wxBLACK_BRUSH );
			l_dc.Clear();
			l_dc.SetTextBackground( *wxBLACK );
			l_dc.SetTextForeground( *wxWHITE );
			l_dc.DrawText( l_chars, 0, 5 );

//			l_bitmap.SaveFile( wxT( "Coin_" ) + wxString( std::to_string( i ) ) + wxT( ".bmp" ), wxBITMAP_TYPE_BMP );

			ByteArray l_buffer( m_iMaxHeight * m_iMaxHeight );
			wxNativePixelData l_data( l_bitmap );
			uint8_t * l_pBuffer = &l_buffer[0];

			if( l_data )
			{
				wxNativePixelData::Iterator l_it( l_data );

				for( int y = 0 ; y < m_iMaxHeight ; ++y )
				{
					wxNativePixelData::Iterator l_itRowStart = l_it;

					for( int x = 0 ; x < m_iMaxHeight ; ++x, ++l_it )
					{
						*l_pBuffer = l_it.Red();l_pBuffer++;
					}

					l_it = l_itRowStart;
					l_it.OffsetY( l_data, 1 );
				}
			}

			m_arrayGlyphs[i].SetSize( wxSize( l_iWidth, m_iMaxHeight ) );
			m_arrayGlyphs[i].SetPosition( wxPoint( 0, 0 ) );
			m_arrayGlyphs[i].SetBitmap( l_buffer );
		}
	}
}

void TextOverlay :: Draw( GlFrameBuffer & p_frameBuffer )
{
	if( m_bBackgroundModified )
	{
		m_textureBackground.Activate();
		m_textureBackground.UploadGlImageSync();
		m_textureBackground.Deactivate();
		m_bBackgroundModified = false;
	}

	if( m_bForegroundModified )
	{
		m_textureForeground.Activate();
		m_textureForeground.UploadGlImageSync();
		m_textureForeground.Deactivate();
		m_bForegroundModified = false;
	}

	m_textureBackground.Initialise( m_size );
	m_textureForeground.Initialise( m_size );
	m_textureText.Initialise( m_size );

	m_textureBackgroundCut.Initialise( m_size );
	m_frameBufferBackgroundCut.Initialise( m_size );

	m_textureBlendedText.Initialise( m_size );
	m_frameBufferBlendedText.Initialise( m_size );

	m_textureResult.Initialise( m_size );
	m_frameBufferResult.Initialise( m_size );

	if( m_bTextModified && m_arrayGlyphs.size() > 0 )
	{
		wxPoint l_ptPos( 0, 0 );
		wxChar c;
		size_t l_uiIndex = 0;
		PixelBuffer & l_pxBuffer = m_textureText.GetBuffer();
		l_pxBuffer.init();

		for( wxString::iterator l_it = m_strText.begin() ; l_it != m_strText.end() ; ++l_it )
		{
			c = *l_it;
			wxSize const & l_sizeChar = m_arrayGlyphs[c].GetSize();
			wxPoint const & l_ptCharPos = m_arrayGlyphs[c].GetPosition();
			ByteArray const & l_buffer = m_arrayGlyphs[c].GetBitmap();
			l_uiIndex = 0;

			if( c == wxT( '\n' ) )
			{
				l_ptPos = wxPoint( 0, l_ptPos.y + m_iMaxHeight );
			}
			else if( c == wxT( '\r' ) )
			{
				l_ptPos = wxPoint( 0, l_ptPos.y );
			}
			else
			{
				if( l_ptPos.x + l_ptCharPos.x + l_sizeChar.x >= m_size.x )
				{
					l_ptPos = wxPoint( 0, l_ptPos.y + m_iMaxHeight );
				}

				if( l_ptPos.y + l_ptCharPos.y + l_sizeChar.y <= m_size.y )
				{
					for( int y = 0 ; y < l_sizeChar.y ; y++ )
					{
						for( int x = 0 ; x < l_sizeChar.x ; x++ )
						{
							uint8_t l_r = l_buffer[l_uiIndex++];

							if( l_r != 0 )
							{
								if( l_ptPos.x + x + l_ptCharPos.x >= 0 )
								{
									l_pxBuffer[(m_size.y - 1) - (l_ptCharPos.y + l_ptPos.y + y)][l_ptPos.x + x + l_ptCharPos.x].Set( l_r, l_r, l_r, l_r);
								}
								else
								{
									l_pxBuffer[(m_size.y - 1) - (l_ptCharPos.y + l_ptPos.y + y)][l_ptPos.x + x].Set( l_r, l_r, l_r, l_r);
								}
							}
						}
					}
				}

				l_ptPos.x += l_sizeChar.x + l_ptCharPos.x;
			}
		}
		
		wxBitmap l_bitmap( m_size, 24 );
		wxNativePixelData l_data( l_bitmap );
		UbPixel * l_pBuffer = l_pxBuffer.ptr();

		if( l_data )
		{
			wxNativePixelData::Iterator l_it( l_data );

			for( int y = 0 ; y < m_iMaxHeight ; ++y )
			{
				wxNativePixelData::Iterator l_itRowStart = l_it;

				for( int x = 0 ; x < m_iMaxHeight ; ++x, ++l_it )
				{
					l_it.Red()		= l_pBuffer->r;
					l_it.Green()	= l_pBuffer->r;
					l_it.Blue()		= l_pBuffer->r;
				}

				l_it = l_itRowStart;
				l_it.OffsetY( l_data, 1 );
			}
		}

//		l_bitmap.SaveFile( wxT( "Coin.bmp" ), wxBITMAP_TYPE_BMP );

		m_textureText.Activate();
		m_textureText.UploadGlImageSync();
		m_textureText.Deactivate();
		m_bTextModified = false;
	}

	if( m_bMatrixModified )
	{
		wxSize const & l_size = p_frameBuffer.GetSize();
		m_matrix[ 0] = m_size.x / GLfloat( l_size.x );
		m_matrix[ 5] = m_size.y / GLfloat( l_size.y );
		m_matrix[12] = m_position.x / GLfloat( l_size.x );
		m_matrix[13] = m_position.y / GLfloat( l_size.y );
		m_bMatrixModified = false;
	}

	m_pOpenGl->ClearColor( 0.0, 0.0, 0.0, 0.0 );
	m_pOpenGl->MatrixMode( GL_MODELVIEW );
	m_pOpenGl->LoadIdentity();

	// First, we blend text with chosen text foreground
	m_frameBufferBlendedText.Activate();
	m_pOpenGl->Clear( GL_COLOR_BUFFER_BIT );
	m_textureForeground.Activate(	GL_TEXTURE0,	GL_REPLACE,	NULL,			NULL );
	m_textureText.Activate(			GL_TEXTURE1,	GL_COMBINE,	&m_blendRgbTxt,	&m_blendAlphaTxt );
	DoSubRender();
	m_textureText.Deactivate(		GL_TEXTURE1 );
	m_textureForeground.Deactivate(	GL_TEXTURE0 );
	m_frameBufferBlendedText.Deactivate();

	m_pOpenGl->Enable( GL_BLEND );
	m_pOpenGl->BlendFunc( GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA );

	// Then we blend the blended text with the background to obtain the full overlay
	m_frameBufferResult.Activate();
	m_pOpenGl->Clear( GL_COLOR_BUFFER_BIT);
//	m_textureBackground.Activate(		GL_TEXTURE0 );
//	m_textureBlendedText.Activate(		GL_TEXTURE1, GL_COMBINE, &m_blendRgbImg, &m_blendAlphaImg );
	m_textureText.Activate(			GL_TEXTURE0 );
	DoSubRender();
	m_textureText.Deactivate(		GL_TEXTURE0 );
//	m_textureBlendedText.Deactivate(	GL_TEXTURE1 );
//	m_textureBackground.Deactivate(		GL_TEXTURE0 );
	m_frameBufferResult.Deactivate();

	// Eventually we render the overlay into the given buffer
	m_pOpenGl->MatrixMode( GL_MODELVIEW );
	m_pOpenGl->LoadIdentity();
	m_pOpenGl->MultMatrix( m_matrix );
	p_frameBuffer.Activate();
	m_textureResult.Activate(	GL_TEXTURE0 );
	DoSubRender();
	m_textureResult.Deactivate(	GL_TEXTURE0 );
	p_frameBuffer.Deactivate();

	m_pOpenGl->Disable( GL_BLEND );
}

void TextOverlay :: SetBackground( wxColour const & p_colour )
{
	PixelBuffer & l_buffer = m_textureBackground.GetBuffer();
	l_buffer.init( Size( m_size.x, m_size.y ) );

	for( int y = 0 ; y < m_size.y ; y++ )
	{
		for( int x = 0 ; x < m_size.x ; x++ )
		{
			l_buffer[y][x].r = p_colour.Red();
			l_buffer[y][x].g = p_colour.Green();
			l_buffer[y][x].b = p_colour.Blue();
			l_buffer[y][x].a = p_colour.Alpha();
		}
	}

	m_bBackgroundModified = true;
}

void TextOverlay :: SetForeground( wxColour const & p_colour )
{
	PixelBuffer & l_buffer = m_textureForeground.GetBuffer();
	l_buffer.init( Size( m_size.x, m_size.y ) );

	for( int y = 0 ; y < m_size.y ; y++ )
	{
		for( int x = 0 ; x < m_size.x ; x++ )
		{
			l_buffer[y][x].r = p_colour.Red();
			l_buffer[y][x].g = p_colour.Green();
			l_buffer[y][x].b = p_colour.Blue();
			l_buffer[y][x].a = p_colour.Alpha();
		}
	}

	m_bForegroundModified = true;
}

void TextOverlay :: SetBackground( wxImage const & p_image )
{
	wxImage l_image = p_image.ResampleBicubic( m_size.x, m_size.y );
	PixelBuffer & l_buffer = m_textureBackground.GetBuffer();
	l_buffer.init( Size( m_size.x, m_size.y ) );

	if( p_image.HasAlpha() )
	{
		l_buffer.set< PG_UINT8_RGBA >( p_image.GetData(), p_image.GetAlpha() );
	}
	else
	{
		l_buffer.set< PG_UINT8_RGB >( p_image.GetData(), NULL );
	}

	m_bBackgroundModified = true;
}

void TextOverlay :: SetForeground( wxImage const & p_image )
{
	wxImage l_image = p_image.ResampleBicubic( m_size.x, m_size.y );
	PixelBuffer & l_buffer = m_textureForeground.GetBuffer();
	l_buffer.init( Size( m_size.x, m_size.y ) );

	if( p_image.HasAlpha() )
	{
		l_buffer.set< PG_UINT8_RGBA >( p_image.GetData(), p_image.GetAlpha() );
	}
	else
	{
		l_buffer.set< PG_UINT8_RGB >( p_image.GetData(), NULL );
	}

	m_bForegroundModified = true;
}

void TextOverlay :: DoSubRender()
{
	if( m_vtxBuffer.Activate() )
	{
		if( m_idxBuffer.Activate() )
		{
			m_pOpenGl->DrawElements( GL_TRIANGLES, m_idxBuffer.GetSize(), GL_UNSIGNED_INT, 0 );
			m_idxBuffer.Deactivate();
		}

		m_vtxBuffer.Deactivate();
	}
}

//*************************************************************************************************

ProceduralGenerator::Thread :: Thread( ProceduralGenerator * p_pParent, size_t p_uiIndex, int iWidth, int iTop, int iBottom, int iTotalHeight, const UbPixel & p_pxColour)
	:	wxThread	( wxTHREAD_DETACHED	)
	,	m_pParent	( p_pParent			)
	,	m_uiIndex	( p_uiIndex			)
	,	m_iWidth	( iWidth			)
	,	m_iBottom	( iBottom			)
	,	m_iTop		( iTop				)
	,	m_iHeight	( iTotalHeight		)
	,	m_pxColour	( p_pxColour		)
	,	m_bStopped	( false				)
	,	m_bLaunched	( false				)
{
}

ProceduralGenerator::Thread :: ~Thread()
{
}

wxThread::ExitCode ProceduralGenerator::Thread :: Entry()
{

	while( !IsStopped() )
	{
		if( IsLaunched() )
		{
			{
				wxMutexLocker l_locker( m_mutex );
				m_bLaunched = false;
			}
			// Lock thread data
			m_mutex.Lock();
			// Make computations
			Step();
			// Unlock thread data
			m_mutex.Unlock();
			// Tell parent I am ended
			wxCommandEvent l_event( wxEVT_NULL, eEVENT_WORKER_THREAD_END );
			l_event.SetInt( int( m_uiIndex ) );
			m_pParent->AddPendingEvent( l_event );
		}

		Sleep( 1 );
	}

	return 0;
}

//*************************************************************************************************

ProceduralGenerator :: ProceduralGenerator( int p_width, int p_height, int WXUNUSED( p_iWndId ), wxFrame * p_pMainFrame, bool p_bNeedsConfigPanel, int p_iSleepTime )
	:	wxThread				( wxTHREAD_DETACHED			)
	,	m_openGl				(							)
	,	m_sizeDisplay			( p_width, p_height			)
	,	m_sizeImage				( p_width, p_height			)
	,	m_step					( 0							)
	,	m_initialised			( false						)
	,	m_pxColour				( 255, 255, 255, 255		)
	,	m_bStopped				( false						)
	,	m_frontBuffer			( Size( p_width, p_height )	)
	,	m_backBuffer			( Size( p_width, p_height )	)
	,	m_pMainFrame			( p_pMainFrame 				)
	,	m_bNeedsConfigPanel		( p_bNeedsConfigPanel		)
	,	m_bToSave				( false						)
	,	m_bRendered				( true						)
	,	m_sleepTime				( p_iSleepTime				)
	,	m_pfnCpuStepEndHandler	( NULL						)
	,	m_texture				( &m_openGl					)
	,	m_frameBuffer			( &m_openGl					)
	,	m_textureOverlays		( &m_openGl					)
	,	m_frameBufferOverlays	( &m_openGl					)
	,	m_textureSave			( &m_openGl					)
	,	m_frameBufferSave		( &m_openGl					)
	,	m_vertexBuffer			( &m_openGl					)
	,	m_indexBuffer			( &m_openGl					)
	,	m_overlayStatus			( &m_openGl					)
{
	m_texture.Resize( m_sizeImage );
	m_frameBuffer.Resize( m_sizeImage );
	m_frameBuffer.BindTexture( GL_COLOR_ATTACHMENT0, &m_texture );

	m_textureOverlays.Resize( m_sizeImage );
	m_frameBufferOverlays.Resize( m_sizeImage );
	m_frameBufferOverlays.BindTexture( GL_COLOR_ATTACHMENT0, &m_textureOverlays );

	m_frameBufferSave.BindTexture( GL_COLOR_ATTACHMENT0, &m_textureSave );
	m_uiThreadCount = GetCPUCount();// - 1;
	Connect( eEVENT_WORKER_THREAD_END,	wxEVT_NULL,		wxCommandEventHandler(	ProceduralGenerator::OnThreadEnd	) );
	Connect( eEVENT_THREAD_STOP,		wxEVT_NULL,		wxCommandEventHandler(	ProceduralGenerator::OnStop 		) );

	Connect( wxEVT_MOTION, wxMouseEventHandler( ProceduralGenerator::OnMouseMove ), NULL, this );

	m_overlayStatus.SetSize( wxSize( p_width, 30 ) );
	m_overlayStatus.SetPosition( wxPoint( 0, p_height - 30 ) );
	m_overlayStatus.Create( 20, wxFONTSTYLE_NORMAL, wxFONTWEIGHT_NORMAL );
}

ProceduralGenerator :: ~ProceduralGenerator()
{
	Disconnect( eEVENT_WORKER_THREAD_END,	wxEVT_NULL,		wxCommandEventHandler(	ProceduralGenerator::OnThreadEnd	) );
	Disconnect( eEVENT_THREAD_STOP,			wxEVT_NULL,		wxCommandEventHandler(	ProceduralGenerator::OnStop 		) );

	Disconnect( wxEVT_MOTION, wxMouseEventHandler( ProceduralGenerator::OnMouseMove ), NULL, this );
}

wxThread::ExitCode ProceduralGenerator :: Entry()
{
	wxMilliClock_t	l_beforeTime;
	wxMilliClock_t	l_time;
	ExitCode 		l_eReturn = 0;

	while( !IsStopped() )
	{
		l_beforeTime = wxGetLocalTimeMillis();

		if( m_initialised && IsRendered() )
		{
			SetRendered( false );
			SwapBuffers();
			InitialiseStep();
			WaitForStepEnd();
			wxCommandEvent 	l_event( wxEVT_NULL, eEVENT_CPU_STEP_END );
			m_pMainFrame->GetEventHandler()->AddPendingEvent( l_event );
		}

		l_time = wxGetLocalTimeMillis() - l_beforeTime;

		if( l_time < m_sleepTime )
		{
			wxMilliSleep( (m_sleepTime - l_time).ToLong() );
		}
		else
		{
			wxMilliSleep( m_sleepTime.ToLong() );
		}
	}

	DoCleanup();

	return l_eReturn;
}

void ProceduralGenerator :: SetRed( uint8_t val )
{
	m_pxColour.r = val;

	for( size_t i = 0 ; i < m_arraySlaveThreads.size() ; i++ )
	{
		if( m_arraySlaveThreads[i] )
		{
			m_arraySlaveThreads[i]->SetRed( val );
		}
	}

	DoSetRed( val );
}

void ProceduralGenerator :: SetGreen( uint8_t val )
{
	m_pxColour.g = val;

	for( size_t i = 0 ; i < m_arraySlaveThreads.size() ; i++ )
	{
		if( m_arraySlaveThreads[i] )
		{
			m_arraySlaveThreads[i]->SetGreen( val );
		}
	}

	DoSetGreen( val );
}

void ProceduralGenerator :: SetBlue( uint8_t val )
{
	m_pxColour.b = val;

	for( size_t i = 0 ; i < m_arraySlaveThreads.size() ; i++ )
	{
		if( m_arraySlaveThreads[i] )
		{
			m_arraySlaveThreads[i]->SetBlue( val );
		}
	}

	DoSetBlue( val );
}

void ProceduralGenerator :: SwapBuffers()
{
	m_frontBuffer = m_backBuffer;
}

void ProceduralGenerator :: InitialiseStep()
{
	DoInitialiseStep();
	m_uiThreadEndedCount = 0;

	for (size_t i = 0 ; i < m_arraySlaveThreads.size() ; i++)
	{
		if (m_arraySlaveThreads[i] != NULL)
		{
			m_arraySlaveThreads[i]->Launch();
		}
	}
}

void ProceduralGenerator :: ClearAllThreads()
{
	m_initialised = false;

	for (size_t i = 0 ; i < m_arraySlaveThreads.size() ; i++)
	{
		ProceduralGenerator::Thread * l_pThread = m_arraySlaveThreads[i];
		m_arraySlaveThreads[i] = NULL;

		if( l_pThread )
		{
			l_pThread->Stop();
		}
	}

	m_arraySlaveThreads.clear();
}

void ProceduralGenerator :: GlInitialise()
{
	m_openGl.Initialise();
	m_openGl.ClearColor( 1.0, 1.0, 1.0, 1.0 );
	m_openGl.ShadeModel( GL_FLAT);

	m_vertexBuffer.Initialise();
	m_indexBuffer.Initialise();

	m_texture.Initialise();
	m_frameBuffer.Initialise();

	m_textureOverlays.Initialise();
	m_frameBufferOverlays.Initialise();

	m_textureSave.Initialise();
	m_frameBufferSave.Initialise();

	m_overlayStatus.Initialise();
	m_overlayStatus.SetText( wxT( "Coin" ) );
	m_overlayStatus.SetBackground( wxColour( 0, 0, 0, 127 ) );
	m_overlayStatus.SetForeground( *wxWHITE );

	DoGlInitialise();
}

void ProceduralGenerator :: GlPreRender()
{
	m_texture.Initialise( &m_frontBuffer );
	m_frameBuffer.Initialise( m_sizeImage );

	m_textureOverlays.Initialise( m_sizeImage );
	m_frameBufferOverlays.Initialise( m_sizeImage );

	m_textureSave.Initialise( m_sizeImage );
	m_frameBufferSave.Initialise( m_sizeImage );

	DoGlPreRender();
}

void ProceduralGenerator :: GlRender( bool p_bChanged)
{
	bool l_bChanged( p_bChanged );
	DoGlRender( l_bChanged );
	m_openGl.Viewport( 0, 0, m_sizeImage.x, m_sizeImage.y);

	if (m_bToSave)
	{
		m_frameBufferSave.Activate();
	}
	else
	{
		m_frameBufferOverlays.Activate();
	}


	m_openGl.MatrixMode( GL_PROJECTION);
	m_openGl.LoadIdentity();
	m_openGl.Ortho( 0.0, 1.0, 0.0, 1.0, 0.0, 1.0);
	m_openGl.MatrixMode( GL_MODELVIEW);
	m_openGl.LoadIdentity();
	m_openGl.Clear( GL_COLOR_BUFFER_BIT);
	m_texture.Activate();

	if (l_bChanged)
	{
		m_texture.UploadGlImageSync();
	}

	DoSubRender();
	m_texture.Deactivate();

	if (m_bToSave)
	{
		DoSaveFrame();
		m_frameBufferSave.Deactivate();
	}
	else
	{
		m_frameBufferOverlays.Deactivate();

//		m_overlayStatus.Draw( m_frameBufferOverlays );

		m_openGl.Viewport( 0, 0, m_sizeDisplay.x, m_sizeDisplay.y);
		m_openGl.MatrixMode( GL_PROJECTION );
		m_openGl.LoadIdentity();
		m_openGl.Ortho( 0.0, 1.0, 0.0, 1.0, 0.0, 1.0 );
		m_openGl.MatrixMode( GL_MODELVIEW );
		m_openGl.LoadIdentity();
		m_openGl.Clear( GL_COLOR_BUFFER_BIT );
		m_textureOverlays.Activate();
		DoSubRender();
		m_textureOverlays.Deactivate();
	}
}

void ProceduralGenerator :: GlPostRender()
{
	DoGlPostRender();
	SetRendered( true );
}

void ProceduralGenerator :: GlCleanup()
{
	m_vertexBuffer.Cleanup();
	m_indexBuffer.Cleanup();

	m_texture.Cleanup();
	m_frameBuffer.Cleanup();

	m_textureOverlays.Cleanup();
	m_frameBufferOverlays.Cleanup();

	m_textureSave.Cleanup();
	m_frameBufferSave.Cleanup();

	m_overlayStatus.Cleanup();

	DoGlCleanup();
	m_openGl.Cleanup();
}

void ProceduralGenerator :: WaitForStepEnd()
{
	std::size_t l_uiThreadCount = m_arraySlaveThreads.size();

	while( m_uiThreadEndedCount < l_uiThreadCount && ! IsStopped() )
	{
		Sleep( 1 );
	}
}

void ProceduralGenerator :: Resize( int p_iWidth, int p_iHeight )
{
	Resize( wxSize( p_iWidth, p_iHeight ) );
}

void ProceduralGenerator :: Resize( const wxSize & p_size )
{
	m_sizeImage = p_size;
	m_sizeDisplay = p_size;
	wxSize l_sizeConfig( 0, 0 );

	if( m_bNeedsConfigPanel )
	{
		l_sizeConfig = wxSize( 150, 0 );
	}

	wxDisplay l_display;
	wxSize l_maxSize = l_display.GetClientArea().GetSize() + m_pMainFrame->GetClientSize() - m_pMainFrame->GetSize() - wxSize( l_sizeConfig.x, 0);
	double l_dRatio = double( m_sizeDisplay.x) / double( m_sizeDisplay.y);
	bool l_bChanged = false;

	l_maxSize = wxSize(	std::min<int>( l_maxSize.x, int( l_maxSize.y * l_dRatio ) ),
						std::min<int>( l_maxSize.y, int( l_maxSize.x / l_dRatio ) ) );

	while (m_sizeDisplay.x > l_maxSize.x || m_sizeDisplay.y > l_maxSize.y)
	{
		if (m_sizeDisplay.x > l_maxSize.x)
		{
			m_sizeDisplay.x = l_maxSize.x;
			m_sizeDisplay.y = int( m_sizeDisplay.x / l_dRatio );
		}
		else if (m_sizeDisplay.y > l_maxSize.y)
		{
			m_sizeDisplay.y = l_maxSize.y;
			m_sizeDisplay.x = int( m_sizeDisplay.y * l_dRatio );
		}

		l_bChanged = true;
	}

	wxCommandEvent l_event( wxEVT_NULL, ePG_EVENT_RESIZE_PARENT );
	l_event.SetExtraLong( m_sizeDisplay.x + l_sizeConfig.x );
	l_event.SetInt( m_sizeDisplay.y );
	m_pMainFrame->GetEventHandler()->AddPendingEvent( l_event );

	m_texture.Resize( m_sizeImage);
	m_frameBuffer.Resize( m_sizeImage);

	m_textureOverlays.Resize( m_sizeImage);
	m_frameBufferOverlays.Resize( m_sizeImage);

	DoResize( p_size );
}

void ProceduralGenerator :: SaveFrame()
{
	m_bToSave = true;
}

void ProceduralGenerator :: ShowMessageBox( wxString const & p_title, wxString const & p_message, int p_flags )
{
	wxCommandEvent l_event( wxEVT_NULL, ePG_EVENT_SHOW_MESSAGE_BOX );
	l_event.SetString( p_title + wxT( "||" ) + p_message );
	l_event.SetInt( p_flags );
	m_pMainFrame->GetEventHandler()->AddPendingEvent( l_event );
}

void ProceduralGenerator :: DoCleanup()
{
	ClearAllThreads();
	m_frontBuffer.clear();
	m_backBuffer.clear();
}

wxSize ProceduralGenerator :: DoLoadImage( const wxString & p_strImagePath, wxImage & p_image, PixelBuffer & p_buffer)
{
	wxSize l_sizeReturn;

	if (p_image.LoadFile( p_strImagePath))
	{
		wxDisplay l_display;
		wxSize l_sizeConfig;

		if( m_bNeedsConfigPanel )
		{
			l_sizeConfig = wxSize( 150, 0 );
		}

		l_sizeReturn.x = p_image.GetWidth();
		l_sizeReturn.y = p_image.GetHeight();

		p_image = p_image.Mirror( false);
		p_buffer.init( Size( l_sizeReturn.x, l_sizeReturn.y));

		if (p_image.HasAlpha())
		{
			p_buffer.set< PG_UINT8_RGBA >( p_image.GetData(), p_image.GetAlpha());
		}
		else
		{
			p_buffer.set< PG_UINT8_RGB >( p_image.GetData(), NULL);
		}
	}
	else
	{
		ShowMessageBox( _( "Image loading error" ), _( "No handler found for this image format" ), wxOK | wxCENTRE | wxICON_ERROR );
	}

	return l_sizeReturn;
}

void ProceduralGenerator :: DoSubRender()
{
	if (m_vertexBuffer.Activate())
	{
		if (m_indexBuffer.Activate())
		{
			m_openGl.DrawElements( GL_TRIANGLES, m_indexBuffer.GetSize(), GL_UNSIGNED_INT, 0);
			m_indexBuffer.Deactivate();
		}

		m_vertexBuffer.Deactivate();
	}
}

void ProceduralGenerator :: DoSaveFrame()
{
	if (m_bToSave)
	{
		wxImage l_image( m_sizeImage.x, m_sizeImage.y, true);
		int x = 0;
		int y;
		m_frameBufferSave.DownloadBuffer( GL_COLOR_ATTACHMENT0, m_frontBuffer);

		for (int i = 0 ; i < m_sizeImage.x ; i++)
		{
			y = m_sizeImage.y - 1;
			for (int j = 0 ; j < m_sizeImage.y ; j++)
			{
				const UbPixel & l_pixel = m_frontBuffer[j][i];
				l_image.SetRGB( x, y, l_pixel.r, l_pixel.g, l_pixel.b);
				y--;
			}
			x++;
		}

		l_image.SaveFile( wxT( "./Image.bmp"), wxBITMAP_TYPE_BMP);
		m_bToSave = false;
	}
}

void ProceduralGenerator :: OnThreadEnd( wxCommandEvent & p_event )
{
	m_uiThreadEndedCount++;
	p_event.Skip();
}

void ProceduralGenerator :: OnStop( wxCommandEvent & p_event )
{
	m_bStopped = true;

	if( m_pfnCpuStepEndHandler )
	{
		m_pMainFrame->GetEventHandler()->Disconnect( eEVENT_CPU_STEP_END, wxEVT_NULL );
	}

	p_event.Skip();
}

void ProceduralGenerator :: OnMouseMove( wxMouseEvent & p_event )
{
	m_ptMousePosition = p_event.GetPosition();
	p_event.Skip();
}

//*************************************************************************************************
