#include "ProceduralGenerator/PrecompiledHeader.h"

#include "ProceduralGenerator/RenderPanel.h"

using namespace ProceduralTextures;

//*************************************************************************************************

RenderPanel :: RenderPanel( wxFrame * p_parent, int * p_attribList, const wxPoint & p_position, const wxSize & p_size)
	:	wxGLCanvas( p_parent, wxID_ANY, p_attribList, p_position, p_size)
	,	m_pContext			( NULL		)
	,	m_bInitialised		( false		)
	,	m_pGenerator		( NULL		)
	,	m_bFullScreen		( false		)
	,	m_pFrame			( p_parent	)
{
	SetBackgroundColour( p_parent->GetBackgroundColour() );
	SetForegroundColour( p_parent->GetForegroundColour() );
	Show();
}

RenderPanel :: ~RenderPanel()
{
}

void RenderPanel :: SetGenerator( ProceduralGenerator * p_pGenerator )
{
	m_bFullScreen = false;

	if( m_pGenerator )
	{
		if( m_pContext )
		{
			wxClientDC l_dc( this );
			m_pContext->SetCurrent( *this );
			m_pGenerator->GlCleanup();
		}

		m_pGenerator->Stop();
	}

	m_pGenerator = p_pGenerator;

	if( m_pGenerator )
	{
		SetSize( p_pGenerator->GetDisplaySize() );
		m_pGenerator->Create();
		m_pGenerator->Run();
	}

	m_bInitialised = false;
}

void RenderPanel :: SaveFrame()
{
	if( m_pGenerator )
	{
		m_pGenerator->SaveFrame();
	}
}

void RenderPanel :: Render()
{
	if( IsShown() && GetParent()->IsShown() )
	{
		wxClientDC l_dc( this );

		if( m_pGenerator && m_pGenerator->IsInitialised() )
		{
			if( !m_pContext )
			{
				m_pContext = new wxGLContext( this, NULL );
			}

			m_pContext->SetCurrent( *this );
			
			if( !m_bInitialised )
			{
				m_pGenerator->GlInitialise();
				m_bInitialised = true;
			}

			m_pGenerator->GlPreRender();
			m_pGenerator->GlRender( true );
			m_pGenerator->GlPostRender();

			SwapBuffers();
		}
		else
		{
			l_dc.SetBrush( *wxBLACK_BRUSH );
			l_dc.DrawRectangle( wxPoint( 0, 0 ), GetClientSize() );
		}
	}
}

BEGIN_EVENT_TABLE( RenderPanel, wxGLCanvas)
	EVT_PAINT(			RenderPanel::OnPaint		)
	EVT_SIZE(			RenderPanel::OnSize			)
	EVT_CLOSE(			RenderPanel::OnClose		)
	EVT_KEY_UP(			RenderPanel::OnKeyUp		)
	EVT_MOTION(			RenderPanel::OnMouseMove	)
	EVT_LEFT_DCLICK(	RenderPanel::OnLeftDClick	)
END_EVENT_TABLE()

void RenderPanel :: OnPaint( wxPaintEvent & p_event )
{
	wxPaintDC l_dc( this );

	if( !m_pGenerator )
	{
		l_dc.SetBrush( *wxBLACK_BRUSH );
		l_dc.DrawRectangle( wxPoint( 0, 0 ), GetClientSize() );
	}

	p_event.Skip();
}

void RenderPanel :: OnSize( wxSizeEvent & p_event )
{
	if( m_pGenerator )
	{
		m_pGenerator->SetDisplaySize( p_event.GetSize() );
	}

	if( !m_pGenerator || !m_pGenerator->IsInitialised() )
	{
		wxClientDC l_dc( this );
		l_dc.SetBrush( *wxBLACK_BRUSH );
		l_dc.DrawRectangle( wxPoint( 0, 0 ), GetClientSize() );
	}

	p_event.Skip();
}

void RenderPanel :: OnClose( wxCloseEvent & p_event )
{
	if( m_pContext )
	{
		if( m_pGenerator )
		{
			wxClientDC l_dc( this );
			m_pContext->SetCurrent( *this );
			m_pGenerator->GlCleanup();
		}

		delete m_pContext;
		m_pContext = NULL;
	}

	p_event.Skip();
}

void RenderPanel :: OnKeyUp( wxKeyEvent & p_event )
{
	if( p_event.GetKeyCode() == WXK_F12 )
	{
		SaveFrame();
	}

	p_event.Skip();
}

void RenderPanel :: OnMouseMove( wxMouseEvent & p_event )
{
	if( m_pGenerator )
	{
		m_pGenerator->AddPendingEvent( p_event );
	}

	p_event.Skip();
}

void RenderPanel :: OnLeftDClick( wxMouseEvent & p_event )
{
	if( m_pGenerator && m_pGenerator->IsInitialised() )
	{
		m_bFullScreen = !m_bFullScreen;
		m_pFrame->ShowFullScreen( m_bFullScreen, wxFULLSCREEN_ALL );
	}

	p_event.Skip();
}
