#include "ShaderWebcam/GeneratorShaderWebcam.h"

#include <cmath>

using namespace ProceduralTextures;

//*************************************************************************************************

Effect :: Effect( OpenGl * p_pOpenGl, wxImage * p_pImage )
	:	m_pImage			( p_pImage	)
	,	m_pShaderProgram	( NULL		)
	,	m_strVertexFile		(			)
	,	m_strFragmentFile	(			)
	,	m_bNewShader		( false		)
	,	m_llStartTime		( 0			)
	,	m_iWidth			( 0			)
	,	m_iHeight			( 0			)
	,	m_pOpenGl			( p_pOpenGl	)
{
	if( p_pImage && p_pImage->IsOk() )
	{
		m_iWidth	= p_pImage->GetWidth();
		m_iHeight	= p_pImage->GetHeight();
	}
}

Effect :: ~Effect()
{
	delete m_pShaderProgram;

	for (size_t i = 0 ; i < m_arrayImages.size() ; i++)
	{
		delete m_arrayImages[i];
	}

	m_arrayImages.clear();
	m_arrayImagesFiles.clear();
}

void Effect :: SetVertexFile( const wxString & p_strPath)
{
	m_strVertexFile = p_strPath;
}

void Effect :: SetFragmentFile( const wxString & p_strPath)
{
	m_strFragmentFile = p_strPath;
}

void Effect :: Compile()
{
	m_bNewShader = true;
}

void Effect :: SetImagePath( size_t p_iImage, const wxString & p_strImagePath)
{
	wxImage * l_pImage = NULL;

	if (p_iImage >= m_arrayImages.size())
	{
		l_pImage = new wxImage();
		m_arrayImages.push_back( l_pImage);
		m_arrayImagesFiles.push_back( p_strImagePath);
	}
	else
	{
		l_pImage = m_arrayImages[p_iImage];
		m_arrayImagesFiles[p_iImage] = p_strImagePath;
	}

	if (l_pImage->LoadFile( p_strImagePath))
	{
		int l_iWidth = l_pImage->GetWidth();
		int l_iHeight = l_pImage->GetHeight();
		std::cout << "Original : " << l_iWidth << "x" << l_iHeight << "\n";

		if (m_iWidth != l_iWidth || m_iHeight != l_iHeight)
		{
			std::cout << "Computed : " << m_iWidth << "x" << m_iHeight << "\n";
			l_pImage->Rescale( m_iWidth, m_iHeight, wxIMAGE_QUALITY_HIGH);
			std::cout << "Scaled : " << l_pImage->GetWidth() << "x" << l_pImage->GetHeight() << "\n";
		}
	}
	else
	{
//		wxMessageBox( wxT( "No handler found for this image format"));
	}
}

void Effect :: Initialise()
{
	if (m_pShaderProgram != NULL)
	{
		m_pShaderProgram->Cleanup();
	}
	else
	{
		m_pShaderProgram = new GlShaderProgram( m_pOpenGl );
	}

	if ( ! m_strVertexFile.empty())
	{
		m_pShaderProgram->CreateObject( eSHADER_OBJECT_TYPE_VERTEX);
		m_pShaderProgram->SetProgramFile( m_strVertexFile, eSHADER_OBJECT_TYPE_VERTEX);
	}

	if ( ! m_strFragmentFile.empty())
	{
		m_pShaderProgram->CreateObject( eSHADER_OBJECT_TYPE_PIXEL);
		m_pShaderProgram->SetProgramFile( m_strFragmentFile, eSHADER_OBJECT_TYPE_PIXEL);
	}

	m_pShaderProgram->Initialise();

	m_pShaderProgram->CreateIntFrameVariable( wxT( "pg_width"		),	0 );
	m_pShaderProgram->CreateIntFrameVariable( wxT( "pg_height"		),	0 );
	m_pShaderProgram->CreateIntFrameVariable( wxT( "pg_time"		),	0 );
	m_pShaderProgram->CreateIntFrameVariable( wxT( "pg_texture"		),	0 );
	m_pShaderProgram->CreateIntFrameVariable( wxT( "pg_modif1"		),	1 );
	m_pShaderProgram->CreateIntFrameVariable( wxT( "pg_modif2"		),	2 );
	m_pShaderProgram->CreateIntFrameVariable( wxT( "pg_modif3"		),	3 );
	m_pShaderProgram->CreateIntFrameVariable( wxT( "pg_sep_type"	),	0 );
	m_pShaderProgram->CreateIntFrameVariable( wxT( "pg_sep_offset"	),	0 );

	m_llStartTime = wxGetLocalTimeMillis();
	m_bNewShader = false;
}

void Effect :: Cleanup()
{
	if (m_pShaderProgram != NULL)
	{
		m_pShaderProgram->Cleanup();
	}
}

bool Effect :: Activate( eSEPARATION p_eSeparationType, int p_iSeparationOffset)
{
	bool l_bReturn = true;

	if (m_pShaderProgram != NULL)
	{
		m_pShaderProgram->SetIntFrameVariableValue( wxT( "pg_width"			),	m_iWidth);
		m_pShaderProgram->SetIntFrameVariableValue( wxT( "pg_height"		),	m_iHeight);
		m_pShaderProgram->SetIntFrameVariableValue( wxT( "pg_sep_type"		),	p_eSeparationType);
		m_pShaderProgram->SetIntFrameVariableValue( wxT( "pg_sep_offset"	),	p_iSeparationOffset);
		m_pShaderProgram->SetIntFrameVariableValue( wxT( "pg_time"			),	( wxGetLocalTimeMillis() - m_llStartTime).ToLong());
		m_pShaderProgram->Activate();
	}

	return l_bReturn;
}

void Effect :: Deactivate()
{
	if (m_pShaderProgram != NULL)
	{
		m_pShaderProgram->Deactivate();
	}
}

wxString Effect :: GetCompilerLog( eSHADER_OBJECT_TYPE p_eType)
{
	wxString l_strReturn;

	if (m_pShaderProgram != NULL)
	{
		GlShaderObject * l_pShader = m_pShaderProgram->GetProgram( p_eType);

		if (l_pShader != NULL)
		{
			l_strReturn = l_pShader->GetCompilerLog();
		}
	}

	return l_strReturn;
}

wxString Effect :: GetLinkerLog()
{
	wxString l_strReturn;

	if (m_pShaderProgram != NULL)
	{
		l_strReturn = m_pShaderProgram->GetLinkerLog();
	}

	return l_strReturn;
}

void Effect :: UpdateImages()
{
	m_iWidth = m_pImage->GetWidth();
	m_iHeight = m_pImage->GetHeight();

	for (size_t i = 0 ; i < m_arrayImages.size() ; i++)
	{
		m_arrayImages[i]->LoadFile( m_arrayImagesFiles[i]);
		m_arrayImages[i]->ResampleBicubic( m_iWidth, m_iHeight);
	}
}

//*************************************************************************************************

ShaderWebcam::Thread :: Thread( ProceduralGenerator * p_pParent, size_t p_uiIndex, int iWidth, int iTop, int iBottom, int iTotalHeight, const UbPixel & p_pxColour )
	:	ProceduralGenerator::Thread( p_pParent, p_uiIndex, iWidth, iTop, iBottom, iTotalHeight, p_pxColour )
	,	m_pImage	( NULL	)
	,	m_pCapture	( NULL	)
	,	m_pBuffer	( NULL	)
{

}

ShaderWebcam::Thread :: ~Thread()
{

}

void ShaderWebcam::Thread :: Initialise( CvCapture * p_pCapture, PixelBuffer * p_pBuffer )
{
	m_pCapture = p_pCapture;
	m_pBuffer = p_pBuffer;
}

void ShaderWebcam::Thread :: Step()
{
	m_pImage = cvQueryFrame( m_pCapture );
	wxSize l_sizeReturn;

	if( m_pImage )
	{
		l_sizeReturn.x = m_pImage->width;
		l_sizeReturn.y = m_pImage->height;

		m_pBuffer->set< PG_UINT8_RGB_MIRROR >( (uint8_t *)m_pImage->imageData, NULL );
	}
}

//*************************************************************************************************

wxString g_sepChoices[] =
{	_( "Not separated"	)
,	_( "Horizontally"	)
,	_( "Vertically"		)
};

wxString g_shaChoices[] =
{	_( "Shader 1"	)
,	_( "New..."		)
};

ShaderWebcam :: ShaderWebcam( int p_width, int p_height, int p_iWndId, wxFrame * p_pFrame)
	:	ProceduralGenerator			( p_width, p_height, p_iWndId, p_pFrame, true, 10						)
	,	m_pCapture					( NULL																	)
	,	m_iSeparationOffset			( 50																	)
	,	m_eSeparationType			( eSEPARATION_NONE														)
	,	m_pSelectedEffect			( NULL																	)
	,	m_specStaticSeparator		( Specifics< eCONTROL_TYPE_STATIC	>( _( "Separation" ) )				)
	,	m_specComboSeparator		( Specifics< eCONTROL_TYPE_COMBO	>( g_sepChoices[0], g_sepChoices )	)
	,	m_specSliderOffset			( Specifics< eCONTROL_TYPE_SLIDER	>( 0, 100, 50 )						)
	,	m_specButtonReset			( Specifics< eCONTROL_TYPE_BUTTON	>( _( "Reset Time" ) )				)
	,	m_specStaticShaders			( Specifics< eCONTROL_TYPE_STATIC	>( _( "Shaders" ) )					)
	,	m_specComboShaders			( Specifics< eCONTROL_TYPE_COMBO	>( wxT( "" ), g_shaChoices )		)
	,	m_specButtonVertexFile		( Specifics< eCONTROL_TYPE_BUTTON	>( _( "Vertex" ) )					)
	,	m_specButtonFragmentFile	( Specifics< eCONTROL_TYPE_BUTTON	>( _( "Fragment" ) )				)
	,	m_specButtonCompile			( Specifics< eCONTROL_TYPE_BUTTON	>( _( "Compile" ) )					)
	,	m_specButtonCompilerLog		( Specifics< eCONTROL_TYPE_BUTTON	>( _( "Compiler log" ) )			)
	,	m_specButtonRemove			( Specifics< eCONTROL_TYPE_BUTTON	>( _( "Remove" ) )					)
	,	m_frameBuffer1				( &m_openGl																)
	,	m_frameBuffer2				( &m_openGl																)
	,	m_texture1					( &m_openGl																)
	,	m_texture2					( &m_openGl																)
{
	m_pCapture = cvCaptureFromCAM( CV_CAP_ANY );
	int l_iWidth = p_width;
	int l_iHeight = p_height;

	if( m_pCapture )
	{
		int l_iFps = int( cvGetCaptureProperty( m_pCapture, CV_CAP_PROP_FPS ) );

		if( l_iFps > 0 )
		{
			m_sleepTime = int( 1000.0 / l_iFps );
		}
		else
		{
			m_sleepTime = 10;
		}

		cvSetCaptureProperty( m_pCapture, CV_CAP_PROP_FRAME_WIDTH, l_iWidth );
		cvSetCaptureProperty( m_pCapture, CV_CAP_PROP_FRAME_HEIGHT, l_iHeight );
		l_iWidth = int( cvGetCaptureProperty( m_pCapture, CV_CAP_PROP_FRAME_WIDTH ) );
		l_iHeight = int( cvGetCaptureProperty( m_pCapture, CV_CAP_PROP_FRAME_HEIGHT ) );
		m_frontBuffer.init( Size( l_iWidth, l_iHeight ) );
		m_backBuffer.init( Size( l_iWidth, l_iHeight ) );
		DoResize( wxSize( l_iWidth, l_iHeight ) );
	}
	else
	{
		ShowMessageBox( _( "Capture device" ), _( "No capture device available" ), wxOK | wxCENTRE | wxICON_INFORMATION );
	}

	m_frameBuffer1.BindTexture( GL_COLOR_ATTACHMENT0, & m_texture1 );
	m_frameBuffer2.BindTexture( GL_COLOR_ATTACHMENT0, & m_texture2 );
	wxSystemOptions::SetOption( wxT( "msw.remap" ), 0 );
	wxInitAllImageHandlers();

	DoGeneratePanel();

	if( m_pCapture )
	{
		m_initialised = EffectAdd();
		ShaderWebcam::Thread * l_pThread = CreateThread< ShaderWebcam::Thread >( l_iWidth, 0, l_iHeight, l_iHeight, m_pxColour );
		l_pThread->Initialise( m_pCapture, &m_backBuffer );
		l_pThread->Run();

		m_initialised = true;
	}
}

ShaderWebcam :: ~ShaderWebcam()
{
	for (size_t i = 0 ; i < m_arrayEffects.size() ; i++)
	{
		delete m_arrayEffects[i];
	}

	cvReleaseCapture( &m_pCapture );
	m_arrayEffects.clear();
	wxImage::CleanUpHandlers();
	m_initialised = false;
	DoCleanup();
}

void ShaderWebcam :: ResetTime()
{
	for (size_t i = 0 ; i < m_arrayEffects.size() ; i++)
	{
		m_arrayEffects[i]->ResetTime();
	}
}

void ShaderWebcam :: DoGlInitialise()
{
	m_texture1.Initialise();
	m_texture2.Initialise();

	m_frameBuffer1.Initialise();
	m_frameBuffer2.Initialise();
}

void ShaderWebcam :: DoGlPreRender()
{
	m_texture1.Initialise( & m_backBuffer);
	m_texture2.Initialise( m_sizeImage);

	m_frameBuffer1.Initialise( m_sizeDisplay );
	m_frameBuffer2.Initialise( m_sizeDisplay );

	for( EffectPtrArray::iterator l_it = m_arrayEffects.begin() ; l_it != m_arrayEffects.end() ; ++l_it)
	{
		if( !(*l_it)->IsInitialised() )
		{
			(*l_it)->Initialise();
		}
	}

	m_openGl.Viewport( 0, 0, m_sizeImage.x, m_sizeImage.y);
}

void ShaderWebcam :: DoGlRender( bool & p_bChanged)
{
	GlFrameBuffer * l_pFrameBufferTexture = & m_frameBuffer1;
	GlFrameBuffer * l_pFrameBufferRender = & m_frameBuffer2;

	m_texture1.Activate();
	m_texture1.UploadGlImage();
	m_texture1.Deactivate();

	m_openGl.MatrixMode( GL_PROJECTION);
	m_openGl.LoadIdentity();
	m_openGl.Ortho( 0.0, 1.0, 0.0, 1.0, 0.0, 1.0);
	m_openGl.MatrixMode( GL_MODELVIEW);
	m_openGl.LoadIdentity();

	for (size_t i = 0 ; i < m_arrayEffects.size() ; i++)
	{
		if (l_pFrameBufferRender->Activate())
		{
			if (EffectActivate( i))
			{
				m_openGl.Clear( GL_COLOR_BUFFER_BIT);
				l_pFrameBufferTexture->GetTexture( GL_COLOR_ATTACHMENT0)->Activate();
				DoSubRender();
				l_pFrameBufferTexture->GetTexture( GL_COLOR_ATTACHMENT0)->Deactivate();
				EffectDeactivate( i);
			}

			l_pFrameBufferRender->Deactivate();
		}

		std::swap( l_pFrameBufferTexture, l_pFrameBufferRender);
	}

	m_openGl.MatrixMode( GL_PROJECTION);
	m_openGl.LoadIdentity();
	m_openGl.Ortho( 0.0, 1.0, 1.0, 0.0, 0.0, 1.0);
	m_openGl.MatrixMode( GL_MODELVIEW);
	m_openGl.LoadIdentity();

	m_frameBuffer.Activate();
	m_openGl.Clear( GL_COLOR_BUFFER_BIT);
	l_pFrameBufferTexture->GetTexture( GL_COLOR_ATTACHMENT0)->Activate();
	DoSubRender();
	l_pFrameBufferTexture->GetTexture( GL_COLOR_ATTACHMENT0)->Deactivate();
	m_frameBuffer.Deactivate();
	p_bChanged = false;
}

void ShaderWebcam :: DoGlPostRender()
{
}

void ShaderWebcam :: DoGlCleanup()
{
	for (size_t i = 0 ; i < m_arrayEffects.size() ; i++)
	{
		m_arrayEffects[i]->Cleanup();
		delete m_arrayEffects[i];
	}

	m_arrayEffects.clear();

	m_frameBuffer1.Cleanup();
	m_frameBuffer2.Cleanup();

	m_texture1.Cleanup();
	m_texture2.Cleanup();
}

bool ShaderWebcam :: EffectAdd()
{
	Effect * l_pEffect = new Effect( &m_openGl, &m_image );
	m_arrayEffects.push_back( l_pEffect );
	return true;
}

bool ShaderWebcam :: EffectRemove( size_t p_uiIndex)
{
	bool l_bReturn = p_uiIndex < m_arrayEffects.size();

	if (l_bReturn)
	{
		Effect * l_pEffect = m_arrayEffects[p_uiIndex];
		std::vector <Effect *>::iterator l_it = m_arrayEffects.begin();
		m_arrayEffects.erase( l_it + p_uiIndex);
		delete l_pEffect;
	}

	return l_bReturn;
}

void ShaderWebcam :: EffectSetVertexFile( size_t p_uiIndex, const wxString & p_strPath)
{
	if (p_uiIndex < m_arrayEffects.size())
	{
		m_arrayEffects[p_uiIndex]->SetVertexFile( p_strPath);
	}
}

void ShaderWebcam :: EffectSetFragmentFile( size_t p_uiIndex, const wxString & p_strPath)
{
	if (p_uiIndex < m_arrayEffects.size())
	{
		m_arrayEffects[p_uiIndex]->SetFragmentFile( p_strPath);
	}
}

void ShaderWebcam :: EffectSetImagePath( size_t p_uiIndex, size_t p_iImage, const wxString & p_strImagePath)
{
	m_initialised = false;

	if (p_uiIndex < m_arrayEffects.size())
	{
		m_arrayEffects[p_uiIndex]->SetImagePath( p_iImage, p_strImagePath);
	}

	m_initialised = true;
}

void ShaderWebcam :: EffectCompile( size_t p_uiIndex)
{
	if (p_uiIndex < m_arrayEffects.size())
	{
		m_arrayEffects[p_uiIndex]->Compile();
	}
}

wxString ShaderWebcam :: EffectGetCompilerLog( size_t p_uiIndex, eSHADER_OBJECT_TYPE p_eType)
{
	wxString l_strReturn;

	if (p_uiIndex < m_arrayEffects.size())
	{
		m_arrayEffects[p_uiIndex]->GetCompilerLog( p_eType);
	}

	return l_strReturn;
}

wxString ShaderWebcam :: EffectGetLinkerLog( size_t p_uiIndex)
{
	wxString l_strReturn;

	if (p_uiIndex < m_arrayEffects.size())
	{
		m_arrayEffects[p_uiIndex]->GetLinkerLog();
	}

	return l_strReturn;
}

bool ShaderWebcam :: EffectActivate( size_t p_uiIndex)
{
	bool l_bReturn = p_uiIndex < m_arrayEffects.size();

	if (l_bReturn)
	{
		l_bReturn = m_arrayEffects[p_uiIndex]->Activate( m_eSeparationType, m_iSeparationOffset);
	}

	return l_bReturn;
}

void ShaderWebcam :: EffectDeactivate( size_t p_uiIndex)
{
	if (p_uiIndex < m_arrayEffects.size())
	{
		m_arrayEffects[p_uiIndex]->Deactivate();
	}
}

void ShaderWebcam :: DoResize( const wxSize & p_size )
{
	m_sizeImage = p_size;
	m_frameBuffer1.Resize( m_sizeImage);
	m_frameBuffer2.Resize( m_sizeImage);
	m_texture1.Resize( m_sizeImage);
	m_texture2.Resize( m_sizeImage);
	
	m_frameBuffer.Resize( m_sizeImage );
	m_frameBufferOverlays.Resize( m_sizeImage );
	m_texture.Resize( m_sizeImage );
	m_textureOverlays.Resize( m_sizeImage );
}

void ShaderWebcam :: DoGeneratePanel()
{
	int iWidth = 150;
	m_arrayControls.push_back( ControlInfos( eCONTROL_TYPE_STATIC,	wxID_ANY,			wxPoint(  10, wxDEFAULT_HEIGHT * 0	), wxSize( iWidth -  20, wxDEFAULT_HEIGHT	), &m_specStaticSeparator,		0,					eEVENT_TYPE_NONE,				NULL,															true	) );
	m_arrayControls.push_back( ControlInfos( eCONTROL_TYPE_COMBO,	eID_SEPTYPE,		wxPoint(  10, wxDEFAULT_HEIGHT * 1	), wxSize( iWidth -  20, wxDEFAULT_HEIGHT	), &m_specComboSeparator,		wxCB_READONLY,		eEVENT_TYPE_COMBOBOX_SELECTED,	wxCommandEventHandler(	ShaderWebcam::OnSepType				),	true	) );
	m_arrayControls.push_back( ControlInfos( eCONTROL_TYPE_SLIDER,	eID_SEPOFFSET,		wxPoint(  10, wxDEFAULT_HEIGHT * 2	), wxSize( iWidth -  20, wxDEFAULT_HEIGHT	), &m_specSliderOffset,			wxSL_HORIZONTAL,	eEVENT_TYPE_SLIDER_UPDATED,		wxCommandEventHandler(	ShaderWebcam::OnSepOffset			),	true	) );
	m_arrayControls.push_back( ControlInfos( eCONTROL_TYPE_BUTTON,	eID_RESETTIME,		wxPoint(  10, wxDEFAULT_HEIGHT * 3	), wxSize( iWidth -  20, wxDEFAULT_HEIGHT	), &m_specButtonReset,			0,					eEVENT_TYPE_BUTTON_CLIKED,		wxCommandEventHandler(	ShaderWebcam::OnResetTime			),	true	) );

	m_arrayControls.push_back( ControlInfos( eCONTROL_TYPE_STATIC,	wxID_ANY,			wxPoint(  10, wxDEFAULT_HEIGHT * 4	), wxSize( iWidth -  20, wxDEFAULT_HEIGHT	), &m_specStaticShaders,		0,					eEVENT_TYPE_NONE,				NULL,															true	) );
	m_arrayControls.push_back( ControlInfos( eCONTROL_TYPE_COMBO,	eID_SHADERS,		wxPoint(  10, wxDEFAULT_HEIGHT * 5	), wxSize( iWidth -  20, wxDEFAULT_HEIGHT	), &m_specComboShaders,			wxCB_READONLY,		eEVENT_TYPE_COMBOBOX_SELECTED,	wxCommandEventHandler(	ShaderWebcam::OnSelectShader		),	true	) );
	m_arrayControls.push_back( ControlInfos( eCONTROL_TYPE_BUTTON,	eID_VERTEXFILE,		wxPoint(  10, wxDEFAULT_HEIGHT * 6	), wxSize( iWidth -  20, wxDEFAULT_HEIGHT	), &m_specButtonVertexFile,		0,					eEVENT_TYPE_LOAD_TEXT_FILEA,	wxCommandEventHandler(	ShaderWebcam::OnVertexShaderPath	),	false	) );
	m_arrayControls.push_back( ControlInfos( eCONTROL_TYPE_BUTTON,	eID_FRAGMENTFILE,	wxPoint(  10, wxDEFAULT_HEIGHT * 7	), wxSize( iWidth -  20, wxDEFAULT_HEIGHT	), &m_specButtonFragmentFile,	0,					eEVENT_TYPE_LOAD_TEXT_FILEB,	wxCommandEventHandler(	ShaderWebcam::OnFragmentShaderPath	),	false	) );
	m_arrayControls.push_back( ControlInfos( eCONTROL_TYPE_BUTTON,	eID_COMPILE,		wxPoint(  10, wxDEFAULT_HEIGHT * 8	), wxSize( iWidth -  20, wxDEFAULT_HEIGHT	), &m_specButtonCompile,		0,					eEVENT_TYPE_BUTTON_CLIKED,		wxCommandEventHandler(	ShaderWebcam::OnShaderCompile		),	false	) );
	m_arrayControls.push_back( ControlInfos( eCONTROL_TYPE_BUTTON,	eID_COMPILERLOG,	wxPoint(  10, wxDEFAULT_HEIGHT * 9	), wxSize( iWidth -  20, wxDEFAULT_HEIGHT	), &m_specButtonCompilerLog,	0,					eEVENT_TYPE_BUTTON_CLIKED,		wxCommandEventHandler(	ShaderWebcam::OnCompilerLog			),	false	) );
	m_arrayControls.push_back( ControlInfos( eCONTROL_TYPE_BUTTON,	eID_REMOVE,			wxPoint(  10, wxDEFAULT_HEIGHT * 10	), wxSize( iWidth -  20, wxDEFAULT_HEIGHT	), &m_specButtonRemove,			0,					eEVENT_TYPE_BUTTON_CLIKED,		wxCommandEventHandler(	ShaderWebcam::OnRemove				),	false	) );
}

void ShaderWebcam :: OnSepType( wxCommandEvent	& p_event)
{
	SetSeparationType( eSEPARATION( p_event.GetInt()));

	if (p_event.GetInt() != 0)
	{
		m_specSliderOffset.m_pControl->Show();
	}
	else
	{
		m_specSliderOffset.m_pControl->Hide();
	}
}

void ShaderWebcam :: OnSepOffset( wxCommandEvent & p_event)
{
	SetSeparationOffset( p_event.GetInt());
}

void ShaderWebcam :: OnResetTime( wxCommandEvent & WXUNUSED( p_event ) )
{
	ResetTime();
}

void ShaderWebcam :: OnSelectShader( wxCommandEvent & p_event )
{
	unsigned int l_uiSelection = (unsigned int)p_event.GetSelection();

	if( l_uiSelection == m_specComboShaders.m_pControl->GetCount() - 1 )
	{
		if( EffectAdd() )
		{
			wxString l_strPage = wxT( "Shader ");
			l_strPage << m_specComboShaders.m_pControl->GetCount();
			m_specComboShaders.m_pControl->SetString( l_uiSelection, l_strPage );
			m_specComboShaders.m_pControl->Append( _( "New..." ) );
			m_specComboShaders.m_pControl->SetSelection( l_uiSelection );
		}
		else
		{
			l_uiSelection = m_specComboShaders.m_pControl->GetCount();
		}
	}
	else if( l_uiSelection > 0 )
	{
		m_specButtonRemove.m_pControl->Show();
	}
	else
	{
		m_specButtonRemove.m_pControl->Hide();
	}

	if( l_uiSelection < m_specComboShaders.m_pControl->GetCount() )
	{
		m_specButtonVertexFile.m_pControl->Show();
		m_specButtonFragmentFile.m_pControl->Show();
		m_pSelectedEffect = m_arrayEffects[l_uiSelection];
	}
	else
	{
		m_specButtonVertexFile.m_pControl->Hide();
		m_specButtonFragmentFile.m_pControl->Hide();
	}
}

void ShaderWebcam :: OnRemove( wxCommandEvent & WXUNUSED( p_event ) )
{
	size_t l_uiToRemove = size_t( m_specComboShaders.m_pControl->GetSelection() );

	if( l_uiToRemove > 0 && l_uiToRemove < m_specComboShaders.m_pControl->GetCount() - 1 )
	{
		if( EffectRemove( l_uiToRemove ) )
		{
			m_specComboShaders.m_pControl->Delete( int( l_uiToRemove ) );
			m_specButtonVertexFile.m_pControl->Hide();
			m_specButtonFragmentFile.m_pControl->Hide();
			m_specButtonCompile.m_pControl->Hide();
			m_specButtonCompilerLog.m_pControl->Hide();
			m_specButtonRemove.m_pControl->Hide();
		}
	}

	m_pSelectedEffect = NULL;
}

void ShaderWebcam :: OnShaderCompile( wxCommandEvent & WXUNUSED( p_event ) )
{
	if( m_pSelectedEffect )
	{
		m_pSelectedEffect->Compile();
		m_specButtonCompile.m_pControl->Hide();
		m_specButtonCompilerLog.m_pControl->Show();
	}
}

void ShaderWebcam :: OnCompilerLog( wxCommandEvent & WXUNUSED( p_event ) )
{
	if( m_pSelectedEffect )
	{
		wxString l_strVertexLog		= m_pSelectedEffect->GetCompilerLog( eSHADER_OBJECT_TYPE_VERTEX	);
		wxString l_strFragmentLog	= m_pSelectedEffect->GetCompilerLog( eSHADER_OBJECT_TYPE_PIXEL	);
		wxString l_strLinkerLog		= m_pSelectedEffect->GetLinkerLog();
		wxString l_strText;

		if ( ! l_strVertexLog.empty())
		{
			l_strText += _( "Vertex Shader :" ) + wxT( "\n" ) + l_strVertexLog;
		}

		if ( ! l_strFragmentLog.empty())
		{
			if ( ! l_strText.empty())
			{
				l_strText += wxT( "\n\n" );
			}

			l_strText += _( "Fragment Shader :" ) + wxT( "\n" ) + l_strFragmentLog + wxT( "\n\n" );
		}

		if ( ! l_strLinkerLog.empty())
		{
			if ( ! l_strText.empty())
			{
				l_strText += wxT( "\n\n" );
			}

			l_strText += _( "Linker :" ) + wxT( "\n" ) + l_strLinkerLog + wxT( "\n\n" );
		}

		if (l_strText.empty())
		{
			l_strText = _( "No error" );
		}

		ShowMessageBox( _( "Compile and Link result" ), l_strText, wxOK | wxCENTRE | wxICON_INFORMATION );
	}
}

void ShaderWebcam :: OnVertexShaderPath( wxCommandEvent & p_event )
{
	wxString l_strFile = p_event.GetString();

	if( m_pSelectedEffect && ! l_strFile.empty() )
	{
		wxString l_strFile = p_event.GetString();
		m_specButtonCompile.m_pControl->Show();
		m_specButtonCompilerLog.m_pControl->Hide();
		m_pSelectedEffect->SetVertexFile( l_strFile );
	}
}

void ShaderWebcam :: OnFragmentShaderPath( wxCommandEvent & p_event )
{
	wxString l_strFile = p_event.GetString();

	if( m_pSelectedEffect && ! l_strFile.empty() )
	{
		wxString l_strFile = p_event.GetString();
		m_specButtonCompile.m_pControl->Show();
		m_specButtonCompilerLog.m_pControl->Hide();
		m_pSelectedEffect->SetFragmentFile( l_strFile );
	}
}

//******************************************************************************

IMPLEMENT_PLUGIN( SwPlugin, ShaderWebcam)

SwPlugin :: SwPlugin()
	:	Plugin< ShaderWebcam, SwPlugin >( wxT( "ShaderWebcam" ) )
{
}

SwPlugin :: ~SwPlugin()
{
}

wxString SwPlugin :: GetName()
{
	wxString l_strBaseName = _( "Shader Webcam" );

#if !defined( NDEBUG )
	l_strBaseName += wxT( " (Debug)" );
#endif

	return l_strBaseName;
}

//*************************************************************************************************
